-- BeneCast by Skurel
-- v 1.152
-- Code heavily referenced from Danboo's Healomatic/CastParty. Thank you Danboo!
-- Makes clickable buttons that casts beneficial spells on party members and the current target

--
-- Constants
--

-- Subframes for the BeneCast Options frame
local MAX_NUMBER_OF_BUTTONS = 10;
local BENECASTFRAME_SUBFRAMES = {'BeneCastPlayerFrame', 'BeneCastClass1Frame', 'BeneCastClass2Frame', 'BeneCastClass3Frame', 'BeneCastClass4Frame', 'BeneCastClass5Frame', 'BeneCastClass6Frame', 'BeneCastClass7Frame', 'BeneCastClass8Frame', 'BeneCastSetupFrame',};

local STRING_CONSTANTS = BeneCast_Constants;

--
-- Global variables
--

-- Array to hold possible targets
local _targets = {[0] = 'player', 'party1', 'party2', 'party3', 'party4', 'target',};

-- Array to assign id numbers based on class
local _classes = {
	[STRING_CONSTANTS.CLASS_DRUID]		= 2,
	[STRING_CONSTANTS.CLASS_MAGE]		= 4,
	[STRING_CONSTANTS.CLASS_HUNTER]		= 3,
	[STRING_CONSTANTS.CLASS_PRIEST]		= 5,
	[STRING_CONSTANTS.CLASS_ROGUE]		= 6,
	[STRING_CONSTANTS.CLASS_WARLOCK]	= 7,
	[STRING_CONSTANTS.CLASS_WARRIOR]	= 8,
	[STRING_CONSTANTS.CLASS_SHAMAN]		= 9,
	[STRING_CONSTANTS.CLASS_PALADIN]	= 9,
}

-- Healing threshold from max
local _threshold = .9;

-- Class of the caster
local _class;

-- Party notification stuff
local _spellbeingcast = nil;
local _spelltarget = nil;
local _spellismax = nil;
local _spellstarted = nil;
local _spellstopped = nil;
local _sentmessage = nil;

-- Time since the last BeneCastMember frame draw
local _drawtime = {};
	_drawtime[0] = GetTime();
	_drawtime[1] = GetTime();
	_drawtime[2] = GetTime();
	_drawtime[3] = GetTime();
	_drawtime[4] = GetTime();
	_drawtime[5] = GetTime();

-- Tables for interpolating health of targets outside the group
local _class_health = {
	[STRING_CONSTANTS.CLASS_DRUID]	= { ['1'] = 30, ['60'] = 3000 },
	[STRING_CONSTANTS.CLASS_MAGE]	= { ['1'] = 30, ['60'] = 2000 },
	[STRING_CONSTANTS.CLASS_HUNTER]	= { ['1'] = 30, ['60'] = 3000 },
	[STRING_CONSTANTS.CLASS_PALADIN]= { ['1'] = 30, ['60'] = 4000 },
	[STRING_CONSTANTS.CLASS_PRIEST]	= { ['1'] = 30, ['60'] = 2000 },
	[STRING_CONSTANTS.CLASS_ROGUE]	= { ['1'] = 30, ['60'] = 3000 },
	[STRING_CONSTANTS.CLASS_SHAMAN]	= { ['1'] = 30, ['60'] = 3000 },
	[STRING_CONSTANTS.CLASS_WARLOCK]= { ['1'] = 30, ['60'] = 2000 },
	[STRING_CONSTANTS.CLASS_WARRIOR]= { ['1'] = 30, ['60'] = 4000 },
};

-- Table for spell buttons
-- has_spell is for determining which spell types are available to caster for making buttons
-- texture holds the texture of the spell type for use of the button
local _spell_buttons = {};
	_spell_buttons['efficient'] = {has_spell = nil, texture = nil};
	_spell_buttons['emergency'] = {has_spell = nil, texture = nil};
	_spell_buttons['group'] = {has_spell = nil, texture = nil};
	_spell_buttons['instant'] = {has_spell = nil, texture = nil};
	_spell_buttons['poison'] = {has_spell = nil, texture = nil};
	_spell_buttons['disease'] = {has_spell = nil, texture = nil};
	_spell_buttons['magic'] = {has_spell = nil, texture = nil};
	_spell_buttons['curse'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff1'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff2'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff3'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff4'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff5'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff6'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff7'] = {has_spell = nil, texture = nil};
	_spell_buttons['buff8'] = {has_spell = nil, texture = nil};
	_spell_buttons['buffparty1'] = {has_spell = nil, texture = nil};
	_spell_buttons['buffparty2'] = {has_spell = nil, texture = nil};
	_spell_buttons['buffparty3'] = {has_spell = nil, texture = nil};
	_spell_buttons['buffparty4'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff1'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff2'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff3'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff4'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff5'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff6'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff7'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff8'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff9'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff10'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff11'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff12'] = {has_spell = nil, texture = nil};
	_spell_buttons['selfbuff13'] = {has_spell = nil, texture = nil};
	_spell_buttons['groupbuff'] = {has_spell = nil, texture = nil};

-- Spell Level for spells, used for determining what can be cast on the target
-- The target level must be >= spell level - 10
local _spell_level = {
	[STRING_CONSTANTS.CLASS_DRUID] = {
		['efficient'] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
		['emergency'] = { 12, 18, 24, 30, 36, 42, 48, 54, 60 },
		['instant'] = { 4, 10, 16, 22, 28, 34, 40, 46, 52, 58 },
		['group'] = { 1, 1, 1, 1 },
		['poison'] = { 1, 1 },
		['curse'] = { 1 },
		['selfbuff1'] = { 1 },
		['selfbuff2'] = { 1, 18, 28, 38, 48, 58 },
		['selfbuff3'] = { 44 },
		['buff1'] = { 1, 10, 20, 30, 40, 50, 60 },
		['buff2'] = { 6, 14, 24, 34, 44, 54 },
		['buff3'] = { 40 },
		['groupbuff'] = { 50, 60 },
	},
	[STRING_CONSTANTS.CLASS_MAGE] = {
		['curse'] = { 1, 1 },
		['selfbuff1'] = { 1, 10, 20, 30, 40, 50, 60 },
		['selfbuff2'] = { 20, 28, 36, 44, 42, 60 },
		['selfbuff3'] = { 1 },
		['selfbuff4'] = { 1 },
		['selfbuff5'] = { 1 },
		['selfbuff6'] = { 1 },
		['selfbuff7'] = { 1 },
		['selfbuff8'] = { 1, 46, 52, 58 },
		['selfbuff9'] = { 1 },
		['selfbuff10'] = { 12 },
		['selfbuff11'] = { 34, 46, 58 },
		['buffparty1'] = { 18, 30, 42, 54 },
		['buffparty2'] = { 12, 24, 36, 48, 60 },
		['buff1'] = { 1, 14, 28, 42, 56 },
		['selfbuff12'] = { 20, 30, 40, 50, 60 },
		['selfbuff13'] = { 22, 32, 42, 52 },
	},
	[STRING_CONSTANTS.CLASS_PALADIN] = {
		['efficient'] = { 1, 1, 1, 1, 1, 1, 1, 1 },
		['emergency'] = { 1, 1, 1, 1, 1, 1 },
		['instant'] = { 1, 1, 1 },
		['poison'] = { 1 },
		['magic'] = { 1 },
		['selfbuff1'] = { 6, 18, 34, 50 },
		['selfbuff2'] = { 16, 26, 36, 46, 56, },
		['selfbuff3'] = { 22 },
		['selfbuff4'] = { 30, 40, 50, 60 },
		['selfbuff5'] = { 1, 10, 18, 26, 34, 42, 50, 58 },
		['selfbuff6'] = { 38, 48, 58 },
		['selfbuff7'] = { 6, 12, 22, 32, 42, 52 },
		['selfbuff8'] = { 1, 30, 40, 50, 60 },
		['selfbuff9'] = { 1, 40, 50, 60 },
		['selfbuff10'] = { 1 },
		['buff1'] = { 4, 12, 22, 32, 42, 52 },
		['buff2'] = { 18 },
		['buff3'] = { 40, 50, 60 },
		['buff4'] = { 1, 30, 40, 50, 60 },
		['buff5'] = { 14, 24, 34, 44, 54 },
		['buff6'] = { 40 },
		['buffparty1'] = { 10, 24, 38, 46, 54 },
		['buffparty2'] = { 1 },
		['buffparty3'] = { 46, 54 },
		['buffparty4'] = { 30 },
	},
	[STRING_CONSTANTS.CLASS_PRIEST] = {
		['efficient'] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
		['emergency'] = { 1, 1, 1, 1, 1, 1, 1 },
		['instant'] = { 8, 14, 20, 26, 32, 38, 44, 50, 56 },
		['group'] = { 1, 1, 1, 1 },
		['disease'] = { 1, 1 },
		['magic'] = { 1, 1 },
		['selfbuff1'] = { 1 },
		['selfbuff2'] = { 1 },
		['selfbuff3'] = { 1, 1, 1, 1, 1, 1, 1 },
		['selfbuff4'] = { 1, 30, 40, 50, 60 },
		['selfbuff5'] = { 8, 20, 30, 40, 50, 60 },
		['selfbuff6'] = { 1 },
		['selfbuff7'] = { 1, 28, 36, 44, 52, 60 },
		['selfbuff8'] = { 12, 20, 30, 40, 50, 60 },
		['buffparty1'] = { 6, 12, 18, 24, 30, 36, 42, 48, 54, 60 },
		['buff1'] = { 30, 42, 54 },
		['buff2'] = { 1, 12, 24, 36, 48, 60 },
		['buff3'] = { 30, 42, 56 },
		['buff4'] = { 20 },
		['groupbuff'] = { 48, 60 },
	},
	[STRING_CONSTANTS.CLASS_SHAMAN] = {
		['efficient'] = { 1, 1, 1, 1, 1, 1, 1, 1, 1 },
		['emergency'] = { 1, 1, 1, 1, 1, 1 },
		['group'] = { 1, 1, 1 },
		['poison'] = { 1 },
		['disease'] = { 1 },
		['selfbuff1'] = { 1 },
		['selfbuff2'] = { 8, 16, 24, 32, 40, 48, 56 },
		['buff1'] = { 22 },
		['buff2'] = { 28 },
	},
	[STRING_CONSTANTS.CLASS_WARLOCK] = {
		['selfbuff1'] = { 1, 10, 20, 30, 40, 50, 60 },
		['selfbuff2'] = { 1 },
		['selfbuff3'] = { 1 },
		['selfbuff4'] = { 1 },
		['selfbuff5'] = { 1 },
		['buff1'] = { 26, 38, 50 },
		['buff2'] = { 32, 42, 52 },
		['buff3'] = { 16 },
	},
};

-- Storage for spell data
local _spell_data = {
	[STRING_CONSTANTS.CLASS_DRUID]		= {},
	[STRING_CONSTANTS.CLASS_MAGE]		= {},
	[STRING_CONSTANTS.CLASS_PALADIN]	= {},
	[STRING_CONSTANTS.CLASS_PRIEST]		= {},
	[STRING_CONSTANTS.CLASS_SHAMAN]		= {},
	[STRING_CONSTANTS.CLASS_WARLOCK]	= {},
};

-- Maping of spell lines to casting categories
local _spell_types = {
	[STRING_CONSTANTS.CLASS_DRUID] = {
		[STRING_CONSTANTS.HEAL_HEALING_TOUCH]			= 'efficient',
		[STRING_CONSTANTS.HEAL_REGROWTH]				= 'emergency',
		[STRING_CONSTANTS.HEAL_REJUVENATION]			= 'instant',
		[STRING_CONSTANTS.HEAL_TRANQUILITY]				= 'group',
		[STRING_CONSTANTS.CURE_POISON_1]				= 'poison',
		[STRING_CONSTANTS.CURE_POISON_2]				= 'poison',
		[STRING_CONSTANTS.CURE_CURSE_2]					= 'curse',
		[STRING_CONSTANTS.SELFBUFF_NATURES_SWIFTNESS]	= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_NATURES_GRASP]		= 'selfbuff2',
		[STRING_CONSTANTS.SELFBUFF_BARKSKIN]			= 'selfbuff3',
		[STRING_CONSTANTS.BUFF_MARK_OF_THE_WILD]		= 'buff1',
		[STRING_CONSTANTS.BUFF_THORNS]					= 'buff2',
		[STRING_CONSTANTS.BUFF_INNERVATE]				= 'buff3',
		[STRING_CONSTANTS.GROUPBUFF_GIFT_OF_THE_WILD]	= 'groupbuff',
	},
	[STRING_CONSTANTS.CLASS_MAGE] = {
		[STRING_CONSTANTS.CURE_CURSE_1]					= 'curse',
		[STRING_CONSTANTS.SELFBUFF_FROST_ARMOR]			= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_ICE_ARMOR]			= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_MANA_SHIELD]			= 'selfbuff2',
		[STRING_CONSTANTS.SELFBUFF_EVOCATION]			= 'selfbuff3',
		[STRING_CONSTANTS.SELFBUFF_PRESENCE_OF_MIND]	= 'selfbuff4',
		[STRING_CONSTANTS.SELFBUFF_ARCANE_POWER]		= 'selfbuff5',
		[STRING_CONSTANTS.SELFBUFF_COMBUSTION]			= 'selfbuff6',
		[STRING_CONSTANTS.SELFBUFF_ICE_BLOCK]			= 'selfbuff7',
		[STRING_CONSTANTS.SELFBUFF_ICE_BARRIER]			= 'selfbuff8',
		[STRING_CONSTANTS.SELFBUFF_COLD_SNAP]			= 'selfbuff9',
		[STRING_CONSTANTS.SELFBUFF_SLOW_FALL]			= 'selfbuff10',
		[STRING_CONSTANTS.SELFBUFF_MAGE_ARMOR]			= 'selfbuff11',
		[STRING_CONSTANTS.BUFF_FIRE_WARD]				= 'selfbuff12',
		[STRING_CONSTANTS.BUFF_FROST_WARD]				= 'selfbuff13',
		[STRING_CONSTANTS.PARTYBUFF_AMPLIFY_MAGIC]		= 'buffparty1',
		[STRING_CONSTANTS.PARTYBUFF_DAMPEN_MAGIC]		= 'buffparty2',
		[STRING_CONSTANTS.BUFF_ARCANE_INTELLECT]		= 'buff1',
	},
	[STRING_CONSTANTS.CLASS_PALADIN] = {
		[STRING_CONSTANTS.HEAL_HOLY_LIGHT]					= 'efficient',
		[STRING_CONSTANTS.HEAL_FLASH_OF_LIGHT]				= 'emergency',
		[STRING_CONSTANTS.HEAL_LAY_ON_HANDS]				= 'instant',
		[STRING_CONSTANTS.CURE_PURIFY_1]					= 'poison',
		[STRING_CONSTANTS.CURE_CLEANSE_1]					= 'magic',
		[STRING_CONSTANTS.SELFBUFF_DIVINE_PROTECTION]		= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_DIVINE_SHIELD]			= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_FURY]			= 'selfbuff2',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_JUSTICE]			= 'selfbuff3',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_LIGHT]			= 'selfbuff4',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_RIGHTEOUSNESS]	= 'selfbuff5',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_WISDOM]			= 'selfbuff6',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_THE_CRUSADER]	= 'selfbuff7',
		[STRING_CONSTANTS.SELFBUFF_SEAL_OF_COMMAND]			= 'selfbuff8',
		[STRING_CONSTANTS.SELFBUFF_HOLY_SHIELD]				= 'selfbuff9',
		[STRING_CONSTANTS.SELFBUFF_DIVINE_FAVOR]			= 'selfbuff10',
		[STRING_CONSTANTS.BUFF_BLESSING_OF_MIGHT]			= 'buff1',
		[STRING_CONSTANTS.BUFF_BLESSING_OF_FREEDOM]			= 'buff2',
		[STRING_CONSTANTS.BUFF_BLESSING_OF_LIGHT]			= 'buff3',
		[STRING_CONSTANTS.BUFF_BLESSING_OF_SANCTUARY]		= 'buff4',
		[STRING_CONSTANTS.BUFF_BLESSING_OF_WISDOM]			= 'buff5',
		[STRING_CONSTANTS.BUFF_BLESSING_OF_KINGS]			= 'buff6',
		[STRING_CONSTANTS.PARTYBUFF_BLESSING_OF_PROTECTION]	= 'buffparty1',
		[STRING_CONSTANTS.PARTYBUFF_BLESSING_OF_SALVATION]	= 'buffparty2',
		[STRING_CONSTANTS.PARTYBUFF_BLESSING_OF_SACRIFICE]	= 'buffparty3',
		[STRING_CONSTANTS.PARTYBUFF_DIVINE_INTERVENTION]	= 'buffparty4',
	},
	[STRING_CONSTANTS.CLASS_PRIEST] = {
		[STRING_CONSTANTS.HEAL_LESSER_HEAL]				= 'efficient',
		[STRING_CONSTANTS.HEAL_HEAL]					= 'efficient',
		[STRING_CONSTANTS.HEAL_GREATER_HEAL]			= 'efficient',
		[STRING_CONSTANTS.HEAL_FLASH_HEAL]				= 'emergency',
		[STRING_CONSTANTS.HEAL_RENEW]					= 'instant',
		[STRING_CONSTANTS.HEAL_PRAYER_OF_HEALING]		= 'group',
		[STRING_CONSTANTS.CURE_DISEASE_1]				= 'disease',
		[STRING_CONSTANTS.CURE_DISEASE_2]				= 'disease',
		[STRING_CONSTANTS.CURE_MAGIC_1]					= 'magic',
		[STRING_CONSTANTS.SELFBUFF_FOCUSED_CASTING]		= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_INNER_FOCUS]			= 'selfbuff2',
		[STRING_CONSTANTS.SELFBUFF_DESPERATE_PRAYER]	= 'selfbuff3',
		[STRING_CONSTANTS.SELFBUFF_ELUNES_GRACE]		= 'selfbuff4',
		[STRING_CONSTANTS.SELFBUFF_FADE]				= 'selfbuff5',
		[STRING_CONSTANTS.SELFBUFF_LEVITATE]			= 'selfbuff6',
		[STRING_CONSTANTS.SELFBUFF_SHADOWGUARD]			= 'selfbuff7',
		[STRING_CONSTANTS.SELFBUFF_INNER_FIRE]			= 'selfbuff8',
		[STRING_CONSTANTS.PARTYBUFF_POWER_WORD_SHIELD]	= 'buffparty1',
		[STRING_CONSTANTS.BUFF_DIVINE_SPIRIT]			= 'buff1',
		[STRING_CONSTANTS.BUFF_POWER_WORD_FORTITUDE]	= 'buff2',
		[STRING_CONSTANTS.BUFF_SHADOW_PROTECTION]		= 'buff3',
		[STRING_CONSTANTS.BUFF_FEAR_WARD]				= 'buff4',
		[STRING_CONSTANTS.GROUPBUFF_PRAYER_OF_FORTITUDE]= 'groupbuff',
	},
	[STRING_CONSTANTS.CLASS_SHAMAN] = {
		[STRING_CONSTANTS.HEAL_HEALING_WAVE]			= 'efficient',
		[STRING_CONSTANTS.HEAL_LESSER_HEALING_WAVE]		= 'emergency',
		[STRING_CONSTANTS.HEAL_CHAIN_HEAL]				= 'group',
		[STRING_CONSTANTS.CURE_POISON_1]				= 'poison',
		[STRING_CONSTANTS.CURE_DISEASE_1]				= 'disease',
		[STRING_CONSTANTS.SELFBUFF_NATURES_SWIFTNESS]	= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_LIGHTNING_SHIELD]	= 'selfbuff2',
		[STRING_CONSTANTS.BUFF_WATER_BREATHING]			= 'buff1',
		[STRING_CONSTANTS.BUFF_WATER_WALKING]			= 'buff2',
	},
	[STRING_CONSTANTS.CLASS_WARLOCK] = {
		[STRING_CONSTANTS.SELFBUFF_DEMON_ARMOR]			= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_DEMON_SKIN]			= 'selfbuff1',
		[STRING_CONSTANTS.SELFBUFF_DARK_PACT]			= 'selfbuff2',
		[STRING_CONSTANTS.SELFBUFF_FEL_DOMINATION]		= 'selfbuff3',
		[STRING_CONSTANTS.SELFBUFF_SOUL_LINK]			= 'selfbuff4',
		[STRING_CONSTANTS.SELFBUFF_AMPLIFY_CURSE]		= 'selfbuff5',
		[STRING_CONSTANTS.BUFF_DETECT_LESSER_INVIS]		= 'buff1',
		[STRING_CONSTANTS.BUFF_DETECT_INVIS]			= 'buff1',
		[STRING_CONSTANTS.BUFF_DETECT_GREATER_INVIS]	= 'buff1',
		[STRING_CONSTANTS.BUFF_SHADOW_WARD]				= 'buff2',
		[STRING_CONSTANTS.BUFF_UNENDING_BREATH]			= 'buff3',
	},
};

--
-- BeneCast Internal Functions
--

-- Resets data tables
function BeneCast_ResetData()

	-- Clear _spell_data
	_spell_data = {
		[STRING_CONSTANTS.CLASS_DRUID]		= {},
		[STRING_CONSTANTS.CLASS_MAGE]		= {},
		[STRING_CONSTANTS.CLASS_PALADIN]	= {},
		[STRING_CONSTANTS.CLASS_PRIEST]		= {},
		[STRING_CONSTANTS.CLASS_SHAMAN]		= {},
		[STRING_CONSTANTS.CLASS_WARLOCK]	= {},
	};
	
	-- Clear _spell_buttons
	for i in _spell_buttons do
		_spell_buttons[i].has_spell = nil;
		_spell_buttons[i].texture = nil;
	end

	-- Reload the spell data
	BeneCast_LoadSpellData();
	
end

-- Function to determine if a unit is in your raid group
function BeneCast_UnitInRaid(unit)

	local name = UnitName(unit)
	
	for i = 1, 40 do
		if ( UnitExists('raid'..i) ) then
			if ( UnitName('raid'..i) == name ) then
				return true;
			end
		end
	end

	return nil;
	
end

--
-- BeneCast tooltip parsing functions
--

-- Functions for parsing healing info from tooltip. Thanks Danboo!
local function parse_integer(text, index)

	local i = 1;
	for integer in string.gfind(text, '%d+') do
		if ( i == index ) then
			return tonumber(integer);
		end
		i = i + 1;
	end

end

-- Pull the first integer
local function pi_first(text)
	return parse_integer(text, 1);
end

-- Pull the second integer
local function pi_second(text)
	return parse_integer(text, 2);
end

-- Mapping of healing spell lines to heal amount parsers. Thanks Danboo!
local _spell_parsers = {
	[STRING_CONSTANTS.CLASS_SHAMAN] = {
		[STRING_CONSTANTS.HEAL_HEALING_WAVE]			= pi_second,
		[STRING_CONSTANTS.HEAL_LESSER_HEALING_WAVE]		= pi_second,
		[STRING_CONSTANTS.HEAL_CHAIN_HEAL]				= pi_second,
	},
	[STRING_CONSTANTS.CLASS_PRIEST] = {
		[STRING_CONSTANTS.HEAL_LESSER_HEAL]			= pi_second,
		[STRING_CONSTANTS.HEAL_HEAL]				= pi_second,
		[STRING_CONSTANTS.HEAL_GREATER_HEAL]		= pi_second,
		[STRING_CONSTANTS.HEAL_FLASH_HEAL]			= pi_second,
		[STRING_CONSTANTS.HEAL_RENEW]				= pi_first,
		[STRING_CONSTANTS.HEAL_PRAYER_OF_HEALING]	= pi_second,
	},
	[STRING_CONSTANTS.CLASS_DRUID] = {
		[STRING_CONSTANTS.HEAL_HEALING_TOUCH]	= pi_second,
		[STRING_CONSTANTS.HEAL_REJUVENATION]	= pi_first,
		[STRING_CONSTANTS.HEAL_REGROWTH]		= pi_second,
		[STRING_CONSTANTS.HEAL_TRANQUILITY]		= pi_first,
	},
	[STRING_CONSTANTS.CLASS_PALADIN] = {
		[STRING_CONSTANTS.HEAL_HOLY_LIGHT]		= pi_second,
		[STRING_CONSTANTS.HEAL_FLASH_OF_LIGHT]	= pi_second,
		[STRING_CONSTANTS.HEAL_LAY_ON_HANDS]	= pi_second,
	},
};

--
-- BeneCast Loading of Checkbuttons
--

-- Load up the spells into the BeneCast Frame
function BeneCast_LoadCheckButtons()

	-- Load the Player's buttons
	-- Iterator for which checkbuttons are free
	local i = 1;
	-- For every button the caster as access to
	for j in _spell_buttons do
		if ( _spell_buttons[j].has_spell ) then
			-- Get the name of the last entry in spell_type j
			local name, rank;
			for k in _spell_data[_class][j] do
				name, rank = GetSpellName(_spell_data[_class][j][k].id, SpellBookFrame.bookType);
			end
			-- Set up the text checkbutton i to have the name of the last spell of spell_type j
			getglobal('BeneCastPlayerFrameButton' .. i .. 'Label'):SetText(name);
			getglobal('BeneCastPlayerFrameButton' .. i):Show();
			i = i + 1;
		end
	end
	
	-- Load the Class buttons
	-- Reset iterator for which checkbuttons are free
	i = 1;
	-- For every button the caster as access to
	for j in _spell_buttons do
		if ( _spell_buttons[j].has_spell ) then
			-- Don't show selfbuffs for party checkbuttons
			if ( not string.find(j, 'selfbuff') ) then
				-- Get the name of the last entry in spell_type j
				local name, rank;
				for k in _spell_data[_class][j] do
					name, rank = GetSpellName(_spell_data[_class][j][k].id, SpellBookFrame.bookType);
				end
				-- Set up for each member in the party
				for k = 1, 8 do
					-- Set up the text checkbutton i to have the name of the last spell of spell_type j
					getglobal('BeneCastClass' .. k .. 'FrameButton' .. i .. 'Label'):SetText(name);
					getglobal('BeneCastClass' .. k .. 'FrameButton' .. i):Show();
				end
				i = i + 1;
			end
		end
	end

end

--
-- BeneCast Spell Loading function
--

-- Fill in the _spell_data table by iterating over spells in spellbook
-- and parsing tool tips for info. Thanks Danboo!
function BeneCast_LoadSpellData()

	-- If the player cannot use BeneCast return
	if ( not _spell_data[_class] ) then
		return;
	end
	
	-- Iterator to move through the spellbook
	local i = 1;

	while true do
		
		-- Spell at position i in the player's spellbook
		local spell_name, spell_rank = GetSpellName(i, SpellBookFrame.bookType);
		
		-- If there is no spell at i, break out of the loop
		if not spell_name then
			do break end
		end
		
		-- Load up the spell_type based on the spell name and class
		local spell_type = _spell_types[_class][spell_name];

		-- If the spell_type exists do stuff
		if ( spell_type ) then
			
			-- Set has_spell for this spell_type as true. Do I need this? Probably not...
			_spell_buttons[spell_type].has_spell = true;
			
			-- Set BeneCast_Tooltip to the spell info
			-- Be sure to clear it first
			BeneCast_TooltipTextLeft2:SetText('');
			BeneCast_TooltipTextLeft4:SetText('');
			BeneCast_Tooltip:SetSpell(i, SpellBookFrame.bookType);
			
			local _,_,mana_cost;
			-- If this spell has a set mana cost set mana_cost to it
			if ( string.find(BeneCast_TooltipTextLeft2:GetText(), '^(%d+) Mana') ) then
				_,_,mana_cost = string.find(BeneCast_TooltipTextLeft2:GetText(), '^(%d+) Mana');
			-- If this spell does not have a set mana cost, but does have a rank set mana_cost to the rank
			-- This is to ensure correct spell ordering
			elseif ( spell_rank ~= '' ) then
				mana_cost = string.find(spell_rank, '%d+');
			-- If this spell does not have a set mana cost or rank set mana_cost to zero
			-- I'm guessing there is no need to order a spell type if it has no ranks
			else
				mana_cost = 0;
			end
			
			local heal_amount;
			-- If the player class is Paladin and the spell is Lay on Hands set heal_amount to zero
			-- This is to ensure that the highest rank is always used when used
			if ( _class == STRING_CONSTANTS.CLASS_PALADIN and spell_type == 'instant' ) then
				heal_amount = 0
			-- If the spell_type is a heal spell then set heal_amount to the amount this spell heals
			elseif ( spell_type == 'efficient' or spell_type == 'emergency' or spell_type == 'instant' or spell_type == 'group' ) then 
				heal_amount = _spell_parsers[_class][spell_name](BeneCast_TooltipTextLeft4:GetText());
			end
			
			-- If there isn't anything in _spell_data[_class][spell_type] yet then create a table for it
			if ( not _spell_data[_class][spell_type] ) then
				_spell_data[_class][spell_type] = {};
			end
			
			local entry;
			-- If the entry type is a heal spell be sure to include rank and max_heal information
			if ( spell_type == 'efficient' or spell_type == 'emergency' or spell_type == 'instant' or spell_type == 'group' ) then
				entry = { name = (spell_name),
					rank = (spell_rank),
					max_heal = tonumber(heal_amount),
					mana = tonumber(mana_cost),
					id = i };
			-- If the entry is a cure you don't need the rank
			elseif ( spelltype == 'poison' or spelltype == 'disease' or spelltype == 'magic' or spelltype == 'curse' ) then
				entry = {name = (spell_name),
					mana = tonumber(mana_cost),
					id = i };
			-- Otherwise it's a buff and include the spell rank if you can
			elseif ( spell_rank ~= '' ) then
				entry = {name = (spell_name),
					rank = (spell_rank),
					mana = tonumber(mana_cost),
					id = i };
			-- If there is no rank, don't put a spell rank into the entry
			else
				entry = {name = (spell_name),
					mana = tonumber(mana_cost),
					id = i };
			end
			
			-- Insert the entry into the appropriate table
			table.insert(_spell_data[_class][spell_type], entry);

		end

		i = i + 1;

	end
	
	-- Sort the spells by mana cost within each spell type
	for spell_type in _spell_data[_class] do
		table.sort(_spell_data[_class][spell_type], function(a,b) return a.mana < b.mana end);
	end
	
	-- If the spell_type does not have a texture associated with it set it
	for spell_type in _spell_data[_class] do
		local texture;
			-- Make sure the texture is for the last entry in the table
			for i in _spell_data[_class][spell_type] do
				texture = GetSpellTexture(_spell_data[_class][spell_type][i].id, SpellBookFrame.bookType);
			end
		_spell_buttons[spell_type].texture = texture;
	end
	
	-- Load Spells in the BeneCastFrame
	BeneCast_LoadCheckButtons();

end

--
-- BeneCast spell choose functions
--

-- Interpolates the health of the given unit (used for those outside the party). Thanks Danboo!
-- Returns an estimated health for a target outside the party
function BeneCast_UnitInterpolateHealth(unit, health_percent)
	
	local class      = UnitClass(unit);
	local level      = UnitLevel(unit);
	local slope      = ( _class_health[class]['60'] - _class_health[class]['1'] ) / 59;
	local health_max = level * slope + _class_health[class]['1'];
	local health     = health_max * health_percent / 100;
	return health, health_max;
	
end

-- Function to Choose the appropriate spell to cast. Thanks Danboo!
-- Returns the spell to be cast
function BeneCast_ChooseSpell(type, target, forcemax, forceoverheal)

	-- Return if the caster does not have spell type
	if ( not _spell_data[_class][type] ) then
		return;
	end
	
	-- Return if the unit does not exist
	if ( not _spell_level[_class][type] ) then
		return;
	end
	
	local mana = UnitMana('player');
	local maxhealth = 0;
	local health = 0;
	-- If the type is group then caculate the total party damage percentage
	if ( type == 'group' ) then
		for i = 0,4 do
			maxhealth = maxhealth + UnitHealthMax(_targets[i]);
			health = health + UnitHealth(_targets[i]);
		end
		maxhealth = maxhealth * _threshold;
	-- Else calculate the damage percentage of the target
	else
		maxhealth = UnitHealthMax(target) * _threshold;
		health = UnitHealth(target);
	end
	-- If the target is not in the party use UnitInterpolateHealth to determine health
	if ( not UnitInParty(target) ) then
		health, maxhealth = BeneCast_UnitInterpolateHealth(target, health);
	end
	-- If forcemax is true, cast as if the target had 0 health
	if ( forcemax ) then
		health = 0;
	end
	
	-- overhealamount is the number of ranks to overheal from the targets current hp
	local overhealamount = 0;
	if ( forceoverheal ) then
		overhealamount = BeneCastState[UnitName('player')].overhealamount;
	end
	
	local spell;
	
	-- If the type is a heal then use the loop that looks at max_heal
	if ( type == 'efficient' or type == 'emergency' or type == 'instant' or type == 'group' ) then
		for i in _spell_data[_class][type] do
			-- Check to see if the player has enough mana and the targets level is enough to be targeted
			-- Level only matters on HoTs
			if ( _spell_data[_class][type][i].mana <= mana and UnitLevel(target) >= (_spell_level[_class][type][i] - 10) ) then
				-- Set spell to the current best cast
				spell = _spell_data[_class][type][i];
				-- If this heal bring the target over the threshold break out of the loop
				if ( _spell_data[_class][type][i].max_heal + health >= maxhealth ) then
					if ( overhealamount == 0 ) then
						do break end
					end
					overhealamount = overhealamount - 1;
				end
			-- Set the spell to the lowest possible cast if spell is not set. Used to announce "Out of Mana" and for tooltips
			elseif ( not spell ) then
				spell = _spell_data[_class][type][i];
			-- Break out of loop early if not enough mana or target is too low of level or too low level for the HoT
			else
				do break end
			end
		end
	-- If type is a cure effect then cast the lowest unless forced to max
	elseif ( type == 'poison' or type == 'disease' or type == 'magic' or type == 'curse' ) then
		for i in _spell_data[_class][type] do
			-- Check to see if the player has enough mana. Pretty sure there's no level limit for this
			if ( _spell_data[_class][type][i].mana <= mana ) then
				spell = _spell_data[_class][type][i];
				-- Cast the lowest cure effect unless forcemax is true
				if ( not forcemax ) then
					do break end
				end
			-- Set the spell to the lowest possible cast if spell is not set. Used to announce "Out of Mana" and for tooltips
			elseif ( not spell ) then
				spell = _spell_data[_class][type][i];
			-- Break out of loop early if not enough mana
			else
				do break end
			end
		end
	-- Else the spell being cast is a buff, cast the highest one possible
	else
		for i in _spell_data[_class][type] do
			-- Check to see if the targets level is enough to be targeted. Mana is not an issue when buffing, always cast the max possible
			if ( UnitLevel(target) >= (_spell_level[_class][type][i] - 10) ) then
				spell = _spell_data[_class][type][i];
			-- Set the spell to the lowest possible cast if spell is not set. Used to announce "Out of Mana" and for tooltips
			elseif ( not spell ) then
				spell = _spell_data[_class][type][i];
			-- Break out of loop early if the target is too low of level. Pretty sure level limits buff strength
			else
				do break end
			end
		end
	end
	
	-- Return the spell to be cast
	return spell;

end

--
-- BeneCast Slash Command Handler
--

function BeneCast_SlashCommandHandler(msg)

	local frame = getglobal('BeneCastFrame');
	BeneCast_FrameToggle('LeftButton');
	
end

--
-- BeneCast Key Binding functions
--

-- Function to toggle showing of all buffs
function BeneCast_ToggleBuffs()

	-- Return if BeneCastState for this player has not been setup
	if ( not BeneCastState ) then
		return;
	end
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	
	-- Toggle showallbuffs
	BeneCastState[UnitName('player')].showallbuffs = not BeneCastState[UnitName('player')].showallbuffs;

end

-- Function to cast spells through a binding
function BeneCast_BindingCast(id)

	-- The default target for the spell is the player
	local target = 'player';
	
	-- If there is a target and it's friendly choose the target as the target for the spell
	if ( UnitExists('target') and not UnitCanAttack('player', 'target') ) then
		target = 'target';
	end
	
	if ( not UnitExists(target) ) then
		return;
	end
	
	local spell;
	local spelltype;
	
	if ( target == 'player' ) then
		if ( not getglobal('BeneCastMember0Button' .. id):IsVisible() ) then
			return;
		end
		spelltype = getglobal('BeneCastMember0Button' .. id .. 'SpellType'):GetText();
	else
		if ( not getglobal('BeneCastMember5Button' .. id):IsVisible() ) then
			return;
		end
		spelltype = getglobal('BeneCastMember5Button' .. id .. 'SpellType'):GetText();
	end
	
	-- If a spell is already targeting stop targeting
	if ( SpellIsTargeting() ) then
		SpellStopTargeting();
	end
	
	-- Always take care of items on cursor with left clicks
	if ( button == 'LeftButton' and CursorHasItem() ) then
		if ( unit == 'player' ) then
			AutoEquipCursorItem();
		else
			DropItemOnUnit(target);
		end
		return;
	end

	-- Determine if the max heal has been forced with the shift key
	local forcemax = IsShiftKeyDown();
	
	-- Determine if the overheal has been forced with the ctrl key
	local forceoverheal = IsControlKeyDown();
	
	-- If maxheal is not true then reverse the functionality of forcemax
	if ( not BeneCastState[UnitName('player')].maxheal ) then
		forcemax = not forcemax;
	end
	
	-- If overheal is true, then reverse the functionality of overheal	
	if ( BeneCastState[UnitName('player')].overheal ) then
		forceoverheal = not forceoverheal;
	end

	spell = BeneCast_ChooseSpell(spelltype, target, forcemax, forceoverheal);
	
	-- In case you're targeting an enemy and cast Dispel Magic
	local retarget_enemy = nil;
	
	-- Try casting the spell if it exists
	if ( spell ) then
		-- Dispel Magic can be used offensively, always target the intended target
		if ( _class == STRING_CONSTANTS.CLASS_PRIEST and spelltype == 'magic' ) then
			retarget_enemy = UnitIsEnemy('target', 'player');
			TargetUnit(target);
		end
		-- Cast the spell
		CastSpell(spell.id, SpellBookFrame.bookType);
		_spellbeingcast = spell;
		_spelltarget = target;
		_spellismax = forcemax;
		SpellTargetUnit(target);
	end
	
	-- Re-target the last enemy if you changed targets due to Dispel Magic
	if ( retarget_enemy ) then
		TargetLastEnemy();
	end
	
end

--
-- BeneCast Party Notification functions
--

-- Function for Party Notification
-- Modified SpeakSpell by Danboo. Thanks Danboo!
-- Returns true if a notification message was sent
function BeneCast_PartyNotify(target, spell, ismax, duration)

	-- If there is no duration, give a duration of 0
	if ( not duration ) then
		duration = 0;
	end
	
	local playername = UnitName('player');
	
	if ( not BeneCastState[playername] ) then
		return nil;
	end
	
	if ( not BeneCastState[playername].notify ) then
		return nil;
	end
	
	-- Don't show time if notifytime is nil or duration is 0
	local time = nil;
	if ( duration > 0 and BeneCastState[playername].notifytime ) then
		time = true;
	end
	
	-- Don't show rank if spell.rank is nil
	local rank = nil;
	if ( spell.rank and spell.rank ~= 'Rank 0' ) then
		rank = true;
	else
		spell.rank = 'Rank 0';
	end
	
	-- If the spell is targeting the player and not allowing self casts then return
	if ( not BeneCastState[playername].notifyshowself and target == 'player' ) then
		return nil;
	end
	
	-- Figure out the spelltype for display purposes
	local type = _spell_types[_class][spell.name];
	
	-- If the type is a heal and if notification for heals is false then return
	if ( type == 'efficient' or type == 'emergency' or type == 'instant' or type == 'group' ) then
		if ( not BeneCastState[playername].notifyheal ) then
			return nil;
		-- else if notification for only max heals and not a max heal then return
		elseif ( BeneCastState[playername].notifymaxheal and not ismax ) then
			return nil;
		end
	-- If the type is a cure and if notification for cures is false then return
	elseif ( type == 'poison' or type == 'disease' or type == 'magic' or type == 'curse' ) then
		if ( not BeneCastState[playername].notifycure ) then
			return nil;
		end
	-- The type is a buff and if notification for buffs is false then return
	else
		if ( not BeneCastState[playername].notifybuff ) then
			return nil;
		end
	end

	duration = string.format('%.1f', duration / 1000);

	local message_text;
	
	-- Set the message text based on what options are checked
	if ( rank and BeneCastState[playername].notifyrank and time ) then
		message_text = STRING_CONSTANTS.NOTIFY_ALL;
	elseif ( rank and BeneCastState[playername].notifyrank ) then
		message_text = STRING_CONSTANTS.NOTIFY_RANK;
	elseif ( time ) then
		message_text = STRING_CONSTANTS.NOTIFY_TIME;
	else
		message_text = STRING_CONSTANTS.NOTIFY;
	end
	
	message_text = string.gsub(message_text, '%%s', spell.name);
	message_text = string.gsub(message_text, '%%t', UnitName(target));
	message_text = string.gsub(message_text, '%%d', duration);
	message_text = string.gsub(message_text, '%%r', spell.rank);

	-- Send the message on the appropriate channel
	if ( BeneCastState[playername].notifyself and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage(message_text);
	elseif ( BeneCastState[playername].notifyparty and UnitExists('party1') ) then
		SendChatMessage(message_text, 'PARTY');
	elseif ( BeneCastState[playername].notifyraid and UnitExists('party1') ) then
		SendChatMessage(message_text, 'RAID');
	elseif ( BeneCastState[playername].notifytarget and target ~= 'player' and UnitIsPlayer(target) ) then
		SendChatMessage(message_text, 'WHISPER', nil, UnitName(target));
	else
		return nil;
	end
	
	return true;

end

-- Function for Party Notification on failures and interruptions
-- Modified SpeakSpell by Danboo. Thanks Danboo!
function BeneCast_PartyNotifyFail(event)

	local playername = UnitName('player');
	
	local message_text;
	
	-- Set the message text based on what event called PartyNotifyFail
	-- SPELLCAST_CANCELED is really SPELLCAST_INTERRUPTED if the interruption was caused by the player
	-- To know if interruptions are caused by the player, SPELLCAST_STOP is called just before SPELLCAST_INTERRUPTED
	if event == 'SPELLCAST_FAILED' then
		message_text = STRING_CONSTANTS.NOTIFY_FAILED;
	elseif event == 'SPELLCAST_INTERRUPTED' then
		message_text = STRING_CONSTANTS.NOTIFY_INTERRUPTED;
	elseif event == 'SPELLCAST_CANCELED' then
		message_text = STRING_CONSTANTS.NOTIFY_CANCELED;
	end

	-- Send the message on the appropriate channel
	if ( BeneCastState[playername].notifyself and DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage(message_text);
	elseif ( BeneCastState[playername].notifyparty and UnitExists('party1') ) then
		SendChatMessage(message_text, 'PARTY');
	elseif ( BeneCastState[playername].notifyraid and UnitExists('party1') ) then
		SendChatMessage(message_text, 'RAID');
	elseif ( BeneCastState[playername].notifytarget and target ~= 'player' and UnitIsPlayer(target) ) then
		SendChatMessage(message_text, 'WHISPER', nil, UnitName(target));
	end

end

--
-- BeneCast Button Functions
--

-- Function used to show a BeneCast button
-- Parameters spell_type, and the first free button available
-- 'Free' buttons are buttons that are not being used
-- Returns true if a button was created
function BeneCast_ShowButton(type, freebutton)

	local made_button = nil;
	-- Figure out who this button belongs too
	local member = _targets[this:GetID()];
	-- Return if BeneCastState for this player has not been loaded
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	-- If the player has the spell, wants it shown, and freebutton is <= BeneCastState[UnitName('player')].numberofbuttons then continue
	if ( _spell_buttons[type].has_spell and ( ( member ~= 'player' and BeneCastState[UnitName('player')][type][_classes[UnitClass(member)]] ) or ( member == 'player' and BeneCastState[UnitName('player')][type][1] ) ) and freebutton <= BeneCastState[UnitName('player')].numberofbuttons ) then
		local button = getglobal(this:GetName() .. 'Button' .. freebutton);
		-- Only show buttons for valid target members
		if ( UnitExists(member) and UnitIsFriend('player', member) and not UnitIsDead(member) and not UnitIsGhost(member) ) then
			-- Set made_button to true
			made_button = true;
			local icon = getglobal(this:GetName() .. 'Button' .. freebutton .. 'Icon');
			local spelltype = getglobal(this:GetName() .. 'Button' .. freebutton .. 'SpellType');
			icon:SetTexture(_spell_buttons[type].texture);
			spelltype:SetText(type);
			button:Show();
		else
		-- Hide the button if target is not valid
			button:Hide();
		end
	end

	return made_button;
	
end

-- Function to hide temporary buttons if they're not to be shown
function BeneCast_HideButton(buttonnum)

	local button = getglobal(this:GetName() .. 'Button' .. buttonnum);
	if ( button:IsShown() ) then
		button:Hide();
	end

end

-- Function used to make a button flash if it should be flashing
function BeneCast_ButtonFlash(spelltype, member, firstfreebutton)

	-- Check to see if this member has the spelltype in effect
	-- Iterator for buffs
	local i = 1;

	local haseffect = nil;
	-- Cycle through the Units Buffs to find if the effect is on
	while true do
		-- Break out of the loop if there is no buff at iterator i
		if ( not UnitBuff(member, i) ) then
			do break end
		end
		-- Set BeneCast_Tooltip to the buff at iterator i be sure to clear the relevant text fields first
		BeneCast_TooltipTextLeft1:SetText('');
		BeneCast_Tooltip:SetUnitBuff(member, i);
		-- If the Tooltip has text on the Left1 it may be a buff player can cast
		if ( BeneCast_TooltipTextLeft1:GetText() ) then
			if ( _spell_types[_class][BeneCast_TooltipTextLeft1:GetText()] == spelltype ) then
				haseffect = true;
				do break end
			end
		end
		i = i + 1;
	end

	if ( haseffect ) then
		getglobal(this:GetName() .. 'Button' .. firstfreebutton).flashing = true;
	else
		getglobal(this:GetName() .. 'Button' .. firstfreebutton).flashing = nil;
	end

end

-- Function used to update buttons for all possible targets
function BeneCast_UpdateButtons()

	-- Figure out who this frame belongs too
	local member = _targets[this:GetID()];
	
	-- Hide this BeneCastMemberFrame and return if this member is not valid
	if ( not UnitExists(member) or not UnitIsFriend('player', member) or UnitIsGhost(member) or UnitIsDead(member) or UnitIsDeadOrGhost(member) or not UnitIsConnected(member) ) then
		this:Hide();
		return;
	end
	
	-- Hide this BeneCastMemberFrame if BeneCastState has not been loaded and return
	if ( not BeneCastState ) then
		this:Hide();
		return;
	end
	
	-- Hide this BeneCastMemberFrame if BeneCastState for this player has not been loaded and return
	if ( not BeneCastState[UnitName('player')] ) then
		this:Hide();
		return;
	end
			
	-- If player is a Druid and is shapeshifted hide all buttons
	if ( _class == STRING_CONSTANTS.CLASS_DRUID ) then
		local shapeshiftnum = GetNumShapeshiftForms();
		local icon, name, active;
		local is_shapeshifted = nil;
		for i = 1,shapeshiftnum do
			icon, name, active = GetShapeshiftFormInfo(i);
			if ( active ) then
				is_shapeshifted = true;
				do break end
			end
		end
		
		if ( is_shapeshifted ) then
			this:Hide();
			return;
		end
	end
	
	-- Otherwise show the buttons. Show the BeneCastMember frame first
	this:Show();
	
	-- Used to find the first button that is not used
	local firstfreebutton = 1;
	
	-- Hide all buttons before redrawing them
	for x = firstfreebutton, MAX_NUMBER_OF_BUTTONS do
		BeneCast_HideButton(x);
	end
	
	-- Show Heal buttons as persistant buttons, return if firstfreebutton is greater than the number of buttons that can be shown
	if ( BeneCast_ShowButton('efficient', firstfreebutton) ) then
		firstfreebutton = firstfreebutton +1;
	end
	if BeneCast_ShowButton('emergency', firstfreebutton) then
		-- If you're a druid, check to see if this spell is already in effect
		if ( _class == STRING_CONSTANTS.CLASS_DRUID ) then
			BeneCast_ButtonFlash('emergency', member, firstfreebutton)
		end
		firstfreebutton = firstfreebutton +1;
	end
	if ( BeneCast_ShowButton('instant', firstfreebutton) ) then
		-- If you're a druid or priest, check to see if this spell is already in effect
		if _class == STRING_CONSTANTS.CLASS_DRUID or _class == STRING_CONSTANTS.CLASS_PRIEST then
			BeneCast_ButtonFlash('instant', member, firstfreebutton)
		end
		firstfreebutton = firstfreebutton +1;
	end

	-- Group heals are handled differently than other heals
	-- Only on the Shaman Group heal does targeting matter
	-- Show up only on player unless class is Shaman
	if ( _class == STRING_CONSTANTS.CLASS_SHAMAN or member == 'player' ) then
		if ( BeneCast_ShowButton('group', firstfreebutton) ) then
			firstfreebutton = firstfreebutton + 1;
		end
	end
	
	-- Check to see if anyone in the party is poisoned, diseased, cursed, or has harmful magic effects
	-- Do this before showing buffs so Weakened Soul is detected before drawing buffs
	-- Iterator for debuffs
	local i = 1;
	
	-- Auto hide the buttons. This is done in case there are multiples of one kind of effect
	local poisonshow = nil;
	local diseaseshow = nil;
	local magicshow = nil;
	local curseshow = nil;
	
	local weakenedsoul = nil;
	
	-- Cycle through the Units Debuffs to find removable effects
	while true do
		-- Break out of the loop if there is no debuff at iterator i
		if ( not UnitDebuff(member, i) ) then
			do break end
		end
		-- Set BeneCast_Tooltip to the debuff at iterator i, be sure to clear the relevant text first
		BeneCast_TooltipTextRight1:SetText('');
		BeneCast_TooltipTextLeft1:SetText('');
		BeneCast_Tooltip:SetUnitDebuff(member, i);
		-- Check to see if this debuff is Weakened Soul and save it for when you get to the buffs
		if ( BeneCast_TooltipTextLeft1:GetText() == STRING_CONSTANTS.AILMENT_WEAKENED_SOUL ) then
			weakenedsoul = true;
		end
		-- If the Tooltip has text on the Right1 it may be removable
		if ( BeneCast_TooltipTextRight1:GetText() ) then
			local effect = BeneCast_TooltipTextRight1:GetText();
			if effect == STRING_CONSTANTS.AILMENT_POISON then
				poisonshow = true;
			elseif ( effect == STRING_CONSTANTS.AILMENT_DISEASE ) then
				-- Not sure if this is needed, previous implimentation may have been broken
				-- Paladins can cure disease but don't have a spell with the 'disease' spelltype
				if ( _class == 'Paladin' ) then
					poisonshow = true;
				end
				diseaseshow = true;
			elseif ( effect == STRING_CONSTANTS.AILMENT_MAGIC ) then
				magicshow = true;
			elseif ( effect == STRING_CONSTANTS.AILMENT_CURSE ) then
				curseshow = true;
			end
		end
		i = i + 1;
	end
	
	-- Show Buff buttons as temporary buttons
	-- Check to see if anyone in the party does not have your buffs

	-- Create buff_shown table for holding possible buffs
	local buff_shown = {};
	-- Set buff_shown for all spelltypes = false
	for j in _spell_buttons do
		local type = j;
		-- Only add buffs to the buff_shown table
		if ( type == 'efficient' or type == 'emergency' or type == 'instant' or type == 'group' or type == 'poison' or type == 'disease' or type == 'magic' or type == 'curse' ) then
		-- Don't add the buttons for selfbuffs on others
		elseif ( string.find(type, 'selfbuff') and member ~= 'player' ) then
		-- Don't add the buffparty buttons for targets outside of the party
		elseif ( string.find(type, 'buffparty') and ( not UnitInParty(member) and not BeneCast_UnitInRaid(member) ) ) then
		-- Don't add groupbuffs for anyone other than the target
		elseif ( string.find(type, 'groupbuff') and member ~= 'target' ) then
		-- Otherwise add the buff to buff_shown
		else
			-- Create an entry for buff_shown
			local entry = {name = type, shown = nil, fade = nil};
			
			-- Insert the entry into the buff_shown table
			table.insert(buff_shown, entry);
		end
	end
	
	-- Sort buff_shown by type so that selfbuffs go before partybuffs go before buffs
	table.sort(buff_shown, function(a,b) return a.name < b.name end);
	
	-- Reset the iterator for buffs
	i = 1;
	
	-- See if the groupbuff is on the intended target
	local showgroupbuff = nil;
	
	-- See if a mage's ward is on the player
	local wardshown = nil;
	
	-- Cycle through the Units Buffs to find if the effect is on
	while true do
		-- Break out of the loop if there is no buff at iterator i
		if ( not UnitBuff(member, i) ) then
			do break end
		end
				
		-- Set BeneCast_Tooltip to the buff at iterator i, be sure to clear the relevant text first
		BeneCast_TooltipTextLeft1:SetText('');
		BeneCast_Tooltip:SetUnitBuff(member, i);
		-- If the Tooltip has text on the Left1 it may be a buff player can cast
		if ( BeneCast_TooltipTextLeft1:GetText() ) then
			local effect = _spell_types[_class][BeneCast_TooltipTextLeft1:GetText()];
			if ( _class == STRING_CONSTANTS.CLASS_PRIEST and effect == 'groupbuff' ) then
				showgroupbuff = true;
			end
			if ( _class == STRING_CONSTANTS.CLASS_DRUID and effect == 'groupbuff' ) then
				showgroupbuff = true;
			end
			if ( _class == STRING_CONSTANTS.CLASS_MAGE and effect == 'selfbuff12' or effect == 'selfbuff13' ) then
				wardshown = true;
			end
			for j in buff_shown do
				-- Check to see if the effect name is the same as a buff
				if ( effect == buff_shown[j].name and not BeneCastState[UnitName('player')].showallbuffs ) then
					-- Set the buff_shown entry to true since member us under the buffs effect
					buff_shown[j].shown = true;
				end
				-- If the effect is on, make it so the button is shaded gray
				if ( effect == buff_shown[j].name and BeneCastState[UnitName('player')].showallbuffs ) then
					buff_shown[j].fade = true;
				end
			end
		end
		i = i + 1;
	end
	
	-- Make buff_shown for PW:F or MotW on the intended target true if they have the groupbuff on them
	if ( _class == STRING_CONSTANTS.CLASS_PRIEST and showgroupbuff and not BeneCastState[UnitName('player')].showallbuffs ) then
		for j in buff_shown do
			if ( buff_shown[j].name == 'buff2' ) then
				buff_shown[j].shown = true;
			end
		end
	end
	if ( _class == STRING_CONSTANTS.CLASS_PRIEST and showgroupbuff and BeneCastState[UnitName('player')].showallbuffs ) then
		for j in buff_shown do
			if ( buff_shown[j].name == 'buff2' ) then
				buff_shown[j].fade = true;
			end
		end
	end
	if ( _class == STRING_CONSTANTS.CLASS_DRUID and showgroupbuff and not BeneCastState[UnitName('player')].showallbuffs ) then
		for j in buff_shown do
			if ( buff_shown[j].name == 'buff1' ) then
				buff_shown[j].shown = true;
			end
		end
	end
	if ( _class == STRING_CONSTANTS.CLASS_DRUID and showgroupbuff and BeneCastState[UnitName('player')].showallbuffs ) then
		for j in buff_shown do
			if ( buff_shown[j].name == 'buff1' ) then
				buff_shown[j].fade = true;
			end
		end
	end
	if ( _class == STRING_CONSTANTS.CLASS_MAGE and wardshown and not BeneCastState[UnitName('player')].showallbuffs ) then
		for j in buff_shown do
			if ( buff_shown[j].name == 'selfbuff12' or buff_shown[j].name == 'selfbuff12' ) then
				buff_shown[j].shown = true;
			end
		end
	end
	if ( _class == STRING_CONSTANTS.CLASS_MAGE and wardshown and BeneCastState[UnitName('player')].showallbuffs ) then
		for j in buff_shown do
			if ( buff_shown[j].name == 'selfbuff12' or buff_shown[j].name == 'selfbuff12' ) then
				buff_shown[j].fade = true;
			end
		end
	end
	
	-- Show buff buttons of buffs not in effect if possible
	for j in buff_shown do
		if ( not buff_shown[j].shown and BeneCast_ShowButton(buff_shown[j].name, firstfreebutton) ) then
			-- Make this button hide if it is for PW:S and the effect Weakened Soul is on the target
			if ( _class == STRING_CONSTANTS.CLASS_PRIEST and buff_shown[j].name == 'buffparty1' and not BeneCastState[UnitName('player')].showallbuffs ) then
				if weakenedsoul then
					getglobal(this:GetName() .. 'Button' .. firstfreebutton):Hide()
					firstfreebutton = firstfreebutton - 1;
				else
					getglobal(this:GetName() .. 'Button' .. firstfreebutton):Show()
				end
			end
			-- Make the button shaded gray if the effect is on the intended target
			if ( buff_shown[j].fade ) then
				getglobal(this:GetName() .. 'Button' .. firstfreebutton .. 'Flash'):Show();
			else
				getglobal(this:GetName() .. 'Button' .. firstfreebutton .. 'Flash'):Hide();
			end
			firstfreebutton = firstfreebutton + 1;
		end
	end
	
	-- Show Cure buttons as temporary buttons
		
	-- Show buttons for effects that are removable once
	if ( poisonshow and BeneCast_ShowButton('poison', firstfreebutton) ) then
		firstfreebutton = firstfreebutton+1;
	end
	if ( diseaseshow and BeneCast_ShowButton('disease', firstfreebutton) ) then
		firstfreebutton = firstfreebutton+1;
	end
	if ( magicshow and BeneCast_ShowButton('magic', firstfreebutton) ) then
		firstfreebutton = firstfreebutton+1;
	end
	if ( curseshow and BeneCast_ShowButton('curse', firstfreebutton) ) then
		firstfreebutton = firstfreebutton+1;
	end

	-- Hide all buttons that shouldn't be shown
	for x = firstfreebutton, BeneCastState[UnitName('player')].numberofbuttons do
		BeneCast_HideButton(x);
	end
	
	-- If there are no buttons to be shown, hide the frame
	if ( firstfreebutton == 1 ) then
		this:Hide();
	end
	
	-- Resize the BeneCastMemberFrame based on the number of buttons
	if ( BeneCastState ) then
		if ( BeneCastState[UnitName('player')] ) then
			local frame = getglobal(this:GetName());
			local buttonsize;
			if ( BeneCastState[UnitName('player')].playerasdefault or member == 'player') then
				buttonsize = BeneCastState[UnitName('player')][1].buttonsize;
			else
				buttonsize = BeneCastState[UnitName('player')][_classes[UnitClass(member)]].buttonsize;
			end
			frame:SetWidth(buttonsize * (firstfreebutton - 1) + 11 + (3 * (firstfreebutton - 2)));
			frame:SetHeight(buttonsize + 11);
		end
	end
	
	-- Do button sizing, i = the member number value = the saved value for the class of the member
	i = this:GetID();
	local value;
	if ( UnitExists(member) and member ~= 'player' ) then
		value = BeneCastState[UnitName('player')][_classes[UnitClass(member)]].buttonsize;
	end
	if ( member == 'player' ) then
		value = BeneCastState[UnitName('player')][1].buttonsize;
	end
	-- Resize the buttons based on the saved size
	if ( UnitExists(member) ) then
		for j = 1,MAX_NUMBER_OF_BUTTONS do
			-- If BeneCastMemberiButtonj exists then
			if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
				local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
				-- Set Button height and width to value
				button:SetHeight(value);
				button:SetWidth(value);
				local buttonborder = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'NormalTexture');
				-- Set the Button border to fit around the button
				buttonborder:SetHeight(value * 2 - (value/36) * 10);
				buttonborder:SetWidth(value * 2 - (value/36) * 10);
				local buttoncooldown = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Cooldown');
				-- Set Cooldown size
				buttoncooldown:SetScale((value / 36) * .75);
				local buttonflash = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Flash');
				-- Set Flash size
				buttonflash:SetHeight(value * 2 - (value/36) * 10);
				buttonflash:SetWidth(value * 2 - (value/36) * 10);
				-- Resize the BeneCastMemberFrame based on the number of buttons shown
				if ( button:IsVisible() ) then
					local frame = getglobal('BeneCastMember' .. i);
					frame:SetWidth(value * j + 11 + (3 * (j - 1)));
					frame:SetHeight(value + 11);
				end
			end
		end
	end
	
	-- Hide and re-show all buttons
	-- Why? I don't know, but it seems to repair the problem with buttons not being able to be clicked on
	-- I find this highly peculiar, as the buttons are of the right strata and level, they're not behind anything
	-- You can still see the buttons, they're not hidden. You just can't click on them if they're in certain areas
	-- This *seems* to fix it
	if ( UnitExists(member) ) then
		for j = 1,(firstfreebutton - 1) do
			-- If BeneCastMemberiButtonj exists then
			if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
				local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
				-- Hide and show the button
				button:Hide();
				button:Show();
			end
		end
	end
	
end

--
-- BeneCastFrameTabTemplate Helper Functions
--

function BeneCastFrame_ShowSubFrame(frameName)

	getglobal("BeneCastPlayerFrame"):Hide();
	getglobal("BeneCastClass1Frame"):Hide();
	getglobal("BeneCastClass2Frame"):Hide();
	getglobal("BeneCastClass3Frame"):Hide();
	getglobal("BeneCastClass4Frame"):Hide();
	getglobal("BeneCastClass5Frame"):Hide();
	getglobal("BeneCastClass6Frame"):Hide();
	getglobal("BeneCastClass7Frame"):Hide();
	getglobal("BeneCastClass8Frame"):Hide();
	getglobal("BeneCastSetupFrame"):Hide();
	
	getglobal(frameName):Show();

end

--
-- BeneCastFrameTabTemplate Game Client Handlers
--

function BeneCastTab_OnClick(id)

	if ( id == 1 ) then
		BeneCastFrame_ShowSubFrame('BeneCastPlayerFrame');
	elseif ( id == 2 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass1Frame');
	elseif ( id == 3 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass2Frame');
	elseif ( id == 4 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass3Frame');
	elseif ( id == 5 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass4Frame');
	elseif ( id == 6 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass5Frame');
	elseif ( id == 7 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass6Frame');
	elseif ( id == 8 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass7Frame');
	elseif ( id == 9 ) then
		BeneCastFrame_ShowSubFrame('BeneCastClass8Frame');
	elseif ( id == 10 ) then
		BeneCastFrame_ShowSubFrame('BeneCastSetupFrame');
	end
	PlaySound('igCharacterInfoTab');
	
	getglobal('BeneCastFrameTab1'):Enable();
	getglobal('BeneCastFrameTab1'):SetChecked(nil);
	getglobal('BeneCastFrameTab2'):Enable();
	getglobal('BeneCastFrameTab2'):SetChecked(nil);
	getglobal('BeneCastFrameTab3'):Enable();
	getglobal('BeneCastFrameTab3'):SetChecked(nil);
	getglobal('BeneCastFrameTab4'):Enable();
	getglobal('BeneCastFrameTab4'):SetChecked(nil);
	getglobal('BeneCastFrameTab5'):Enable();
	getglobal('BeneCastFrameTab5'):SetChecked(nil);
	getglobal('BeneCastFrameTab6'):Enable();
	getglobal('BeneCastFrameTab6'):SetChecked(nil);
	getglobal('BeneCastFrameTab7'):Enable();
	getglobal('BeneCastFrameTab7'):SetChecked(nil);
	getglobal('BeneCastFrameTab8'):Enable();
	getglobal('BeneCastFrameTab8'):SetChecked(nil);
	getglobal('BeneCastFrameTab9'):Enable();
	getglobal('BeneCastFrameTab9'):SetChecked(nil);
	getglobal('BeneCastFrameTab10'):Enable();
	getglobal('BeneCastFrameTab10'):SetChecked(nil);
	
	this:Disable()
	this:SetChecked(1);
	
end

--
-- BeneCastCheckButtonTemplate Game Client Handlers
--

-- Sets CheckButtonid to be checked or unchecked
-- Parameters id is the id of the CheckButton, isChecked is true if the checkbutton is checked false otherwise
function BeneCast_SetChecked(isChecked, parentid)

	local type = _spell_types[_class][getglobal(this:GetName() .. 'Label'):GetText()];
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	if ( BeneCastState[UnitName('player')].playerasdefault and parentid == 1 ) then
		for i = 1, 9 do
			BeneCastState[UnitName('player')][type][i] = isChecked;
		end
	else
		BeneCastState[UnitName('player')][type][parentid] = isChecked;
	end
	
end

-- Returns if CheckButtonid has been checked or not
-- Parameter id is the id of the CheckButton
function BeneCast_GetChecked(parentid)

	local type = _spell_types[_class][getglobal(this:GetName() .. 'Label'):GetText()];
	if ( not BeneCastState ) then
		return;
	end
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	
	return BeneCastState[UnitName('player')][type][parentid];

end

--
-- BeneCastSetupCheckButtonTemplate Game Client Handlers
--

-- Sets a setup button to the checked value
function BeneCast_SetSetupButton(isChecked)

	local playername = UnitName('player');
	
	local name = this:GetName();
	
	if ( name == 'BeneCastShiftMaxHealCheckButton' ) then
		BeneCastState[playername].maxheal = isChecked;	
	elseif ( name == 'BeneCastOverHealCheckButton' ) then
		BeneCastState[playername].overheal = isChecked;
	elseif ( name == 'BeneCastShowAllBuffsCheckButton' ) then
		BeneCastState[playername].showallbuffs = isChecked;
		TargetUnit('player');
		ClearTarget();
	elseif ( name == 'BeneCastNotficationCheckButton' ) then
		BeneCastState[playername].notify = isChecked;
	-- Have the Channel check buttons act like Radio Buttons
	elseif ( name == 'BeneCastSelfNotifyCheckButton' ) then
		BeneCastState[playername].notifyself = isChecked;
		if ( isChecked ) then
			BeneCastState[playername].notifyparty = nil;
			BeneCastState[playername].notifyraid = nil;
			BeneCastState[playername].notifytarget = nil;
		end
	elseif ( name == 'BeneCastPartyNotifyCheckButton' ) then
		BeneCastState[playername].notifyparty = isChecked;
		if ( isChecked ) then
			BeneCastState[playername].notifyself = nil;
			BeneCastState[playername].notifyraid = nil;
			BeneCastState[playername].notifytarget = nil;
		end
	elseif ( name == 'BeneCastRaidNotifyCheckButton' ) then
		BeneCastState[playername].notifyraid = isChecked;
		if ( isChecked ) then
			BeneCastState[playername].notifyself = nil;
			BeneCastState[playername].notifyparty = nil;
			BeneCastState[playername].notifytarget = nil;
		end
	elseif ( name == 'BeneCastTargetNotifyCheckButton' ) then
		BeneCastState[playername].notifytarget = isChecked;
		if ( isChecked ) then
			BeneCastState[playername].notifyself = nil;
			BeneCastState[playername].notifyparty = nil;
			BeneCastState[playername].notifyraid = nil;
		end
	elseif ( name == 'BeneCastNotifyShowSelfCheckButton' ) then
		BeneCastState[playername].notifyshowself = isChecked;
	elseif ( name == 'BeneCastNotifyRankCheckButton' ) then
		BeneCastState[playername].notifyrank = isChecked;
	elseif ( name == 'BeneCastNotifyTimeCheckButton' ) then
		BeneCastState[playername].notifytime = isChecked;
	elseif ( name == 'BeneCastNotifyHealCheckButton' ) then
		BeneCastState[playername].notifyheal = isChecked;
	elseif ( name == 'BeneCastNotifyMaxHealCheckButton' ) then
		BeneCastState[playername].notifymaxheal = isChecked;
	elseif ( name == 'BeneCastNotifyCureCheckButton' ) then
		BeneCastState[playername].notifycure = isChecked;
	elseif ( name == 'BeneCastNotifyBuffCheckButton' ) then
		BeneCastState[playername].notifybuff = isChecked;
	elseif ( name == 'BeneCastSelfAsTemplateCheckButton' ) then
		BeneCastState[playername].playerasdefault = isChecked;
	elseif ( name == 'BeneCastShowTooltipsCheckButton' ) then
		BeneCastState[playername].tooltips = isChecked;
	elseif ( name == 'BeneCastShowTooltipsNameCheckButton' ) then
		BeneCastState[playername].tooltipnameonly = isChecked;
	elseif ( name == 'BeneCastUnlockCheckButton' ) then
		BeneCastState[playername].unlock = isChecked;
		if ( isChecked ) then
			getglobal('BeneCastMember0').locked = nil;
			getglobal('BeneCastMember1').locked = nil;
			getglobal('BeneCastMember2').locked = nil;
			getglobal('BeneCastMember3').locked = nil;
			getglobal('BeneCastMember4').locked = nil;
			getglobal('BeneCastMember5').locked = nil;
			getglobal('BeneCastMinimapButton').locked = nil;
		else
			getglobal('BeneCastMember0').locked = 1;
			getglobal('BeneCastMember1').locked = 1;
			getglobal('BeneCastMember2').locked = 1;
			getglobal('BeneCastMember3').locked = 1;
			getglobal('BeneCastMember4').locked = 1;
			getglobal('BeneCastMember5').locked = 1;
			getglobal('BeneCastMinimapButton').locked = 1;
		end
	elseif ( name == 'BeneCastShowMinimapButtonCheckButton' ) then
		BeneCastState.hideminimapbutton = isChecked;
		-- Hide the BeneCastMinimapButton if checked, Show if unchecked
		if ( isChecked ) then
			getglobal('BeneCastMinimapButton'):Hide();
		else
			getglobal('BeneCastMinimapButton'):Show();
		end
	end
	
	this:GetParent():Hide();
	this:GetParent():Show();
	
end

-- Returns if a setup button is checked
function BeneCast_GetSetupButton()

	if ( not BeneCastState ) then
		return;
	end
	
	local playername = UnitName('player');
	
	local name = this:GetName();
	
	local buttontext = getglobal(name .. 'Label');
	
	local isChecked;
	
	if ( name == 'BeneCastShiftMaxHealCheckButton' ) then
		isChecked = BeneCastState[playername].maxheal;
		buttontext:SetText(STRING_CONSTANTS.TEXT_DAMAGE_BASED_HEALS)
		this.tooltipText = STRING_CONSTANTS.TEXT_DAMAGE_BASED_HEALS_TOOLTIP;
	elseif ( name == 'BeneCastNotficationCheckButton' ) then
		isChecked = BeneCastState[playername].notify;
		buttontext:SetText(STRING_CONSTANTS.TEXT_PARTY_NOTIFICATION);
		this.tooltipText = STRING_CONSTANTS.TEXT_PARTY_NOTIFICATION_TOOLTIP;
	elseif ( name == 'BeneCastOverHealCheckButton' ) then
		isChecked = BeneCastState[playername].overheal;
		buttontext:SetText(STRING_CONSTANTS.TEXT_OVERHEAL);
		this.tooltipText = STRING_CONSTANTS.TEXT_OVERHEAL_TOOLTIP;
	elseif ( name == 'BeneCastShowAllBuffsCheckButton' ) then
		isChecked = BeneCastState[playername].showallbuffs;
		buttontext:SetText(STRING_CONSTANTS.TEXT_SHOWALLBUFFS);
		this.tooltipText = STRING_CONSTANTS.TEXT_SHOWALLBUFFS_TOOLTIP;
	elseif ( name == 'BeneCastSelfNotifyCheckButton' ) then
		isChecked = BeneCastState[playername].notifyself;
		buttontext:SetText(STRING_CONSTANTS.TEXT_SELF_CHANNEL);
		this.tooltipText = STRING_CONSTANTS.TEXT_SELF_CHANNEL_TOOLTIP;
	elseif ( name == 'BeneCastPartyNotifyCheckButton' ) then
		isChecked = BeneCastState[playername].notifyparty;
		buttontext:SetText(STRING_CONSTANTS.TEXT_PARTY_CHANNEL);
		this.tooltipText = STRING_CONSTANTS.TEXT_PARTY_CHANNEL_TOOLTIP;
	elseif ( name == 'BeneCastRaidNotifyCheckButton' ) then
		isChecked = BeneCastState[playername].notifyraid;
		buttontext:SetText(STRING_CONSTANTS.TEXT_RAID_CHANNEL);
		this.tooltipText = STRING_CONSTANTS.TEXT_RAID_CHANNEL_TOOLTIP;
	elseif ( name == 'BeneCastTargetNotifyCheckButton' ) then
		isChecked = BeneCastState[playername].notifytarget;
		buttontext:SetText(STRING_CONSTANTS.TEXT_WHISPER_TARGET);
		this.tooltipText = STRING_CONSTANTS.TEXT_WHISPER_TARGET_TOOLTIP;
	elseif ( name == 'BeneCastNotifyShowSelfCheckButton' ) then
		isChecked = BeneCastState[playername].notifyshowself;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_SELF_CASTS);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_SELF_CASTS_TOOLTIP;
	elseif ( name == 'BeneCastNotifyRankCheckButton' ) then
		isChecked = BeneCastState[playername].notifyrank;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_RANK);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_RANK_TOOLTIP;
	elseif ( name == 'BeneCastNotifyTimeCheckButton' ) then
		isChecked = BeneCastState[playername].notifytime;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_TIME);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_TIME_TOOLTIP;
	elseif ( name == 'BeneCastNotifyHealCheckButton' ) then
		isChecked = BeneCastState[playername].notifyheal;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_HEALS);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_HEALS_TOOLTIP;
	elseif ( name == 'BeneCastNotifyMaxHealCheckButton' ) then
		isChecked = BeneCastState[playername].notifymaxheal;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_MAX_HEALS);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_MAX_HEALS_TOOLTIP;
	elseif ( name == 'BeneCastNotifyCureCheckButton' ) then
		isChecked = BeneCastState[playername].notifycure;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_CURES);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_CURES_TOOLTIP;
	elseif ( name == 'BeneCastNotifyBuffCheckButton' ) then
		isChecked = BeneCastState[playername].notifybuff;
		buttontext:SetText(STRING_CONSTANTS.TEXT_NOTIFY_BUFFS);
		this.tooltipText = STRING_CONSTANTS.TEXT_NOTIFY_BUFFS_TOOLTIP;
	elseif ( name == 'BeneCastSelfAsTemplateCheckButton' ) then
		isChecked = BeneCastState[playername].playerasdefault;
		buttontext:SetText(STRING_CONSTANTS.TEXT_PLAYER_AS_DEFAULT);
		this.tooltipText = STRING_CONSTANTS.TEXT_PLAYER_AS_DEFAULT_TOOLTIP;
	elseif ( name == 'BeneCastShowTooltipsCheckButton' ) then
		isChecked = BeneCastState[playername].tooltips;
		buttontext:SetText(STRING_CONSTANTS.TEXT_TOOLTIPS);
		this.tooltipText = STRING_CONSTANTS.TEXT_TOOLTIPS_TOOLTIP;
	elseif ( name == 'BeneCastShowTooltipsNameCheckButton' ) then
		isChecked = BeneCastState[playername].tooltipnameonly;
		buttontext:SetText(STRING_CONSTANTS.TEXT_TOOLTIPS_NAME);
		this.tooltipText = STRING_CONSTANTS.TEXT_TOOLTIPS_NAME_TOOLTIP;
	elseif ( name == 'BeneCastUnlockCheckButton' ) then
		isChecked = BeneCastState[playername].unlock;
		buttontext:SetText(STRING_CONSTANTS.TEXT_UNLOCK_BUTTONS);
		this.tooltipText = STRING_CONSTANTS.TEXT_UNLOCK_BUTTONS_TOOLTIP;
		if ( isChecked ) then
			getglobal('BeneCastMember0').locked = nil;
			getglobal('BeneCastMember1').locked = nil;
			getglobal('BeneCastMember2').locked = nil;
			getglobal('BeneCastMember3').locked = nil;
			getglobal('BeneCastMember4').locked = nil;
			getglobal('BeneCastMember5').locked = nil;
			getglobal('BeneCastMinimapButton').locked = nil;
		else
			getglobal('BeneCastMember0').locked = 1;
			getglobal('BeneCastMember1').locked = 1;
			getglobal('BeneCastMember2').locked = 1;
			getglobal('BeneCastMember3').locked = 1;
			getglobal('BeneCastMember4').locked = 1;
			getglobal('BeneCastMember5').locked = 1;
			getglobal('BeneCastMinimapButton').locked = 1;
		end
	elseif ( name == 'BeneCastShowMinimapButtonCheckButton' ) then
		isChecked = BeneCastState.hideminimapbutton;
		buttontext:SetText(STRING_CONSTANTS.TEXT_HIDE_MINIMAP_BUTTON);
		this.tooltipText = STRING_CONSTANTS.TEXT_HIDE_MINIMAP_BUTTON_TOOLTIP;
		if ( isChecked ) then
			getglobal('BeneCastMinimapButton'):Hide();
		else
			getglobal('BeneCastMinimapButton'):Show();
		end
	end
	
	-- Disable child buttons of Party Notification
	if ( name == 'BeneCastSelfNotifyCheckButton' or name == 'BeneCastPartyNotifyCheckButton' or name == 'BeneCastRaidNotifyCheckButton' or name =='BeneCastTargetNotifyCheckButton' or name == 'BeneCastNotifyShowSelfCheckButton' or name == 'BeneCastNotifyRankCheckButton' or name == 'BeneCastNotifyTimeCheckButton' or name == 'BeneCastNotifyHealCheckButton' or name == 'BeneCastNotifyCureCheckButton' or name == 'BeneCastNotifyBuffCheckButton' ) then
		if ( not BeneCastState[playername].notify ) then
			buttontext:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			this:Disable();
		else
			buttontext:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			this:Enable();
		end
	end
	-- Disable child buttons of Allow Heal
	if ( name == 'BeneCastNotifyMaxHealCheckButton' ) then
		if ( not BeneCastState[playername].notifyheal or not BeneCastState[playername].notify ) then
			buttontext:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			this:Disable();
		else
			buttontext:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			this:Enable();
		end
	end
	-- Disable child buttons of Show Tooltips
	if ( name == 'BeneCastShowTooltipsNameCheckButton' ) then
		if ( not BeneCastState[playername].tooltips ) then
			buttontext:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			this:Disable();
		else
			buttontext:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			this:Enable();
		end
	end
	
	return isChecked;

end

--
-- BeneCastButtonTemplate Helper Functions
--

-- GetSpellId based on texture
-- Returns a spell based on the texture given
function BeneCast_GetButtonSpell(spelltype)

	local spell;
	
	-- Learn what spell it is by using ChooseSpell with spelltype
	spell = BeneCast_ChooseSpell(spelltype, 'player', true, nil);
	
	-- Return if the spell doesn't exist otherwise return the spell
	if ( not spell ) then
		return;
	else
		return spell;
	end

end

--
-- BeneCastButtonTemplate Game Client Handlers
--

-- OnLoad function for BeneCast buttons
function BeneCastButton_OnLoad()
	
	this:RegisterEvent("SPELL_UPDATE_COOLDOWN");
	this.flashing = nil;

end

function BeneCastButton_OnShow()

	-- Do this to ensure buttons are on top of background
	this:SetFrameLevel(this:GetParent():GetFrameLevel()+1);
	getglobal(this:GetName() .. 'Cooldown'):SetFrameLevel(this:GetFrameLevel()+1);

end

-- Do the Cooldown animation for a button
function BeneCastButton_UpdateCooldown()

	-- If the unit for this button does not exist return
	if ( not UnitExists(_targets[this:GetParent():GetID()]) ) then
		return;
	end
	
	-- No need to update for a hidden button
	if ( not this:IsVisible() ) then
		return;
	end
	
	local spell;
	-- Figure out this button's spell type
	local spelltype = getglobal(this:GetName() .. 'SpellType'):GetText();
	
	spell = BeneCast_ChooseSpell(spelltype, 'player', 'true', nil);
	
	if ( not spell ) then
		return;
	end
	
	-- Cooldown stuff that I'm not quite sure how exactly it all works
	local cooldown = getglobal(this:GetName()..'Cooldown');
	local iconTexture = getglobal(this:GetName()..'Icon');
	local start, duration, enable = GetSpellCooldown(spell.id, SpellBookFrame.bookType);
	CooldownFrame_SetTimer(cooldown, start, duration, enable);
	if ( enable == 1 ) then
		iconTexture:SetVertexColor(1.0, 1.0, 1.0);
	else
		iconTexture:SetVertexColor(0.4, 0.4, 0.4);
	end
	
end

-- Do the Flash animation for a button
function BeneCastButton_OnUpdate(elapsed)

	local buffAlphaValue;
	if ( this.flashing ) then
		if ( BuffFrameFlashState == 1 ) then
			buffAlphaValue = (BUFF_FLASH_TIME_ON - BuffFrameFlashTime) / BUFF_FLASH_TIME_ON;
			buffAlphaValue = buffAlphaValue * (1 - BUFF_MIN_ALPHA) + BUFF_MIN_ALPHA;
		else
			buffAlphaValue = BuffFrameFlashTime / BUFF_FLASH_TIME_ON;
			buffAlphaValue = (buffAlphaValue * (1 - BUFF_MIN_ALPHA)) + BUFF_MIN_ALPHA;
			this:SetAlpha(BuffFrameFlashTime / BUFF_FLASH_TIME_ON);
		end
		this:SetAlpha(buffAlphaValue);
	else
		this:SetAlpha(1);
	end
	
end

-- Function used to cast a spell when you click on a BeneCast Button
function BeneCastButton_OnClick(button)

	local target = _targets[this:GetParent():GetID()];
	-- In case you switch targets because you're targeting a friendly
	local retarget_friend = nil;
	
	if ( not UnitExists(target) ) then
		return;
	end
	
	local spell;
	local spelltype = getglobal(this:GetName() .. 'SpellType'):GetText();
	
	-- Target the intended target if the current target is friendly but not the intended target
	if ( not UnitCanAttack('player', 'target') and not UnitIsUnit('target', target) and not string.find(spelltype, 'selfbuff') and UnitExists('target') ) then
		retarget_friend = UnitName('target');			
		TargetUnit(target);
	end
	
	-- If a spell is already targeting stop targeting
	if ( SpellIsTargeting() ) then
		SpellStopTargeting();
	end
	
	-- Always take care of items on cursor with left clicks
	if ( button == 'LeftButton' and CursorHasItem() ) then
		if ( unit == 'player' ) then
			AutoEquipCursorItem();
		else
			DropItemOnUnit(target);
		end
		return;
	end

	-- Determine if the max heal has been forced with the shift key
	local forcemax = IsShiftKeyDown();
	
	-- Determine if the overheal has been forced with the ctrl key
	local forceoverheal = IsControlKeyDown();
	
	-- If maxheal is not true then reverse the functionality of forcemax
	if ( not BeneCastState[UnitName('player')].maxheal ) then
		forcemax = not forcemax;
	end
	
	-- If overheal is true, then reverse the functionality of overheal	
	if ( BeneCastState[UnitName('player')].overheal ) then
		forceoverheal = not forceoverheal;
	end

	spell = BeneCast_ChooseSpell(spelltype, target, forcemax, forceoverheal);
	
	-- In case you're targeting an enemy and cast Dispel Magic
	local retarget_enemy = nil;
	
	-- Try casting the spell if it exists
	if ( spell ) then
		-- Dispel Magic can be used offensively, always target the intended target
		if ( _class == STRING_CONSTANTS.CLASS_PRIEST and spelltype == 'magic' ) then
			retarget_enemy = UnitIsEnemy('target', 'player');
			TargetUnit(target);
		end
		-- Cast the spell
		CastSpell(spell.id, SpellBookFrame.bookType);
		_spellbeingcast = spell;
		_spelltarget = target;
		_spellismax = forcemax;
		SpellTargetUnit(target);
	end
	
	-- Re-target the last enemy if you changed targets due to Dispel Magic
	if ( retarget_enemy ) then
		TargetLastEnemy();
	end
	
	-- Re-target the previous friendly target if you changed targets
	if ( retarget_friend ) then
		TargetByName(retarget_friend);
	end
	
end

-- Function for BeneCast button tooltips when mousing over them
function BeneCastButton_OnEnter()

	if ( BeneCastState[UnitName('player')].tooltips ) then
		GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetSpell(BeneCast_GetButtonSpell(getglobal(this:GetName() .. 'SpellType'):GetText()).id, SpellBookFrame.bookType);
	end
	if ( BeneCastState[UnitName('player')].tooltips and BeneCastState[UnitName('player')].tooltipnameonly ) then
		GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(BeneCast_GetButtonSpell(getglobal(this:GetName() .. 'SpellType'):GetText()).name);
	end

end

--
-- BeneCastMemberTemplate Helper Functions
--

function BeneCastMember_SetLock()

	local lock = getglobal('BeneCastMember' .. this.value);
	if ( lock.locked ) then
		lock.locked = nil;
	else
		lock.locked = 1;
	end

end

function BeneCastMemberDropDown_Initialize()

	local info;
	-- Add Title Button
	info = {};
	info.text = UnitName(_targets[this:GetID()]);
	info.isTitle = 1;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);
	
	-- Add Lock Button
	info = {};
	info.text = STRING_CONSTANTS.TEXT_LOCKED;
	info.value = this:GetID();
	info.checked = this.locked;
	info.func = BeneCastMember_SetLock;
	UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL);

end

--
-- BeneCastMemberTemplate Game Event Handlers
--

-- Initializes DropDown menus for BeneCastMember frames
function BeneCastMemberDropDown_OnLoad()

	UIDropDownMenu_Initialize(this, BeneCastMemberDropDown_Initialize, 'MENU');

end

-- OnLoad function for BeneCastMembers
function BeneCastMember_OnLoad()
	
	-- set our default colors
	this:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b);
	this:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b);
	this.locked = 1;
	this.time = 0;
	this:SetFrameLevel(4);
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	this:RegisterEvent("SPELLS_CHANGED");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterForDrag("LeftButton");
	this:RegisterForClicks("RightButtonUp");
	
end

-- OnEvent for BeneCastMember frames, called once for every BeneCastMember frame (6)
function BeneCastMember_OnEvent(event, arg1)

	-- Return if the player cannot use BeneCast
	if ( not _spell_types[_class] ) then
		return;
	
	-- Don't bother with UNIT_AURA if it is for an enemy or for someone other than it's respective BeneCastMember frame
	elseif ( event == 'UNIT_AURA' and not UnitCanAttack('player', arg1) and arg1 == _targets[this:GetID()] ) then
		-- Return if the time between now and the last draw time is less than a second
		if ( GetTime() - _drawtime[this:GetID()] < .05 ) then
			return;
		else
			BeneCast_UpdateButtons();
			_drawtime[this:GetID()] = GetTime();
		end
	elseif ( event == 'PARTY_MEMBERS_CHANGED' ) then
		BeneCast_UpdateButtons();
	elseif ( event == 'SPELLS_CHANGED' ) then
		BeneCast_UpdateButtons();
	elseif ( event == 'PLAYER_TARGET_CHANGED' ) then
		BeneCast_UpdateButtons();
	elseif ( event == 'UNIT_MODEL_CHANGED' ) then
		BeneCast_UpdateButtons();
		if ( BeneCastState ) then
			if ( BeneCastState[UnitName('player')] ) then
				this.locked = not BeneCastState[UnitName('player')].unlock;
			end
		end
	end
	
end

--
-- BeneCastButtonSizeSlider Game Client Handlers
--

-- Sets up the text for a the button size sliders
function BeneCast_ButtonSizeSliderLoad()

	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this.tooltipText = STRING_CONSTANTS.TEXT_BUTTONSIZESLIDER_TOOLTIP;
	getglobal(this:GetName() .. 'ButtonSizeSliderText'):SetText(STRING_CONSTANTS.TEXT_BUTTON_SIZE);
	getglobal(this:GetName() .. 'ButtonSizeSliderLow'):SetText(STRING_CONSTANTS.TEXT_BUTTON_SIZE_LOW);
	getglobal(this:GetName() .. 'ButtonSizeSliderHigh'):SetText(STRING_CONSTANTS.TEXT_BUTTON_SIZE_HIGH);
	
end

-- Resizes BeneCast buttons
function BeneCast_ButtonResize(parentID)

	-- Return if BeneCastState or the player name has not been loaded
	if ( not UnitName('player') or not BeneCastState ) then
		return;
	end
	
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	
	-- Set value to the slider value
	local value = this:GetValue();
	-- For this BeneCastMember, or all members if playerasdefault
	local MemberMax = 5;
	local MemberMin = 0;
	-- If using the player as default then size this slider like the player's
	if ( BeneCastState[UnitName('player')].playerasdefault ) then
		if ( parentID ~= 1 ) then
			value = BeneCastState[UnitName('player')][1].buttonsize;
		end
		-- Hide this slider if using the player's options as the default
		if ( parentID ~= 1 ) then
			this:Hide();
		end
	else
		this:Show();
	end
	for i = MemberMin, MemberMax do
		-- For all buttons
		for j = 1,MAX_NUMBER_OF_BUTTONS do
			-- Don't do anything to button j if the member at i is not a of the same class
			if ( UnitExists(_targets[i]) and _targets[i] ~= 'player' ) then
				if ( parentID == _classes[UnitClass(_targets[i])] and not BeneCastState[UnitName('player')].playerasdefault and _targets[i] ~= 'player' ) then
					-- If BeneCastMemberiButtonj exists then
					if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
						local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
						-- Set Button height and width to value
						button:SetHeight(value);
						button:SetWidth(value);
						local buttonborder = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'NormalTexture');
						-- Set the Button border to fit around the button
						buttonborder:SetHeight(value * 2 - (value/36) * 10);
						buttonborder:SetWidth(value * 2 - (value/36) * 10);
						local buttoncooldown = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Cooldown');
						-- Set Cooldown size
						buttoncooldown:SetScale((value / 36) * .75);
						local buttonflash = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Flash');
						-- Set Flash size
						buttonflash:SetHeight(value * 2 - (value/36) * 10);
						buttonflash:SetWidth(value * 2 - (value/36) * 10);
						-- Resize the BeneCastMemberFrame based on the number of buttons shown
						if ( button:IsVisible() ) then
							local frame = getglobal('BeneCastMember' .. i);
							frame:SetWidth(value * j + 11 + (3 * (j - 1)));
							frame:SetHeight(value + 11);
						end
					end
				elseif ( parentID == 1 and BeneCastState[UnitName('player')].playerasdefault ) then
					-- If BeneCastMemberiButtonj exists then
					if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
						local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
						-- Set Button height and width to value
						button:SetHeight(value);
						button:SetWidth(value);
						local buttonborder = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'NormalTexture');
						-- Set the Button border to fit around the button
						buttonborder:SetHeight(value * 2 - (value/36) * 10);
						buttonborder:SetWidth(value * 2 - (value/36) * 10);
						local buttoncooldown = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Cooldown');
						-- Set Cooldown size
						buttoncooldown:SetScale((value / 36) * .75);
						local buttonflash = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Flash');
						-- Set Flash size
						buttonflash:SetHeight(value * 2 - (value/36) * 10);
						buttonflash:SetWidth(value * 2 - (value/36) * 10);
						-- Resize the BeneCastMemberFrame based on the number of buttons shown
						if ( button:IsVisible() ) then
							local frame = getglobal('BeneCastMember' .. i);
							frame:SetWidth(value * j + 11 + (3 * (j - 1)));
							frame:SetHeight(value + 11);
						end
					end
				end
			elseif ( parentID == 1 and _targets[i] == 'player' ) then
				if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
					local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
					-- Set Button height and width to value
					button:SetHeight(value);
					button:SetWidth(value);
					local buttonborder = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'NormalTexture');
					-- Set the Button border to fit around the button
					buttonborder:SetHeight(value * 2 - (value/36) * 10);
					buttonborder:SetWidth(value * 2 - (value/36) * 10);
					local buttoncooldown = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Cooldown');
					-- Set Cooldown size
					buttoncooldown:SetScale((value / 36) * .75);
					local buttonflash = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Flash');
					-- Set Flash size
					buttonflash:SetHeight(value * 2 - (value/36) * 10);
					buttonflash:SetWidth(value * 2 - (value/36) * 10);
					-- Resize the BeneCastMemberFrame based on the number of buttons shown
					if ( button:IsVisible() ) then
						local frame = getglobal('BeneCastMember' .. i);
						frame:SetWidth(value * j + 11 + (3 * (j - 1)));
						frame:SetHeight(value + 11);
					end
				end
			end
		end
	end
	-- Save the new value into BeneCastState
	BeneCastState[UnitName('player')][parentID].buttonsize = value;
	-- Save the new value into all buttonsizes if playerasdefault
	if ( BeneCastState[UnitName('player')].playerasdefault ) then
		BeneCastState[UnitName('player')][1].buttonsize = value;
		BeneCastState[UnitName('player')][2].buttonsize = value;
		BeneCastState[UnitName('player')][3].buttonsize = value;
		BeneCastState[UnitName('player')][4].buttonsize = value;
		BeneCastState[UnitName('player')][5].buttonsize = value;
		BeneCastState[UnitName('player')][6].buttonsize = value;
		BeneCastState[UnitName('player')][7].buttonsize = value;
		BeneCastState[UnitName('player')][8].buttonsize = value;
		BeneCastState[UnitName('player')][9].buttonsize = value;
	end
	
end

-- Loads BeneCast button size
function BeneCast_ButtonSizeLoad(parentID)

	-- Return if BeneCastState or the player name has not been loaded
	if ( not UnitName('player') or not BeneCastState ) then
		return;
	end
	
	-- Return if BeneCastState['player'] has not yet been created
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	
	-- Set value to the saved size in BeneCastState
	local value = BeneCastState[UnitName('player')][parentID].buttonsize;
	
	-- For this BeneCastMember, or all members if playerasdefault
	-- For all buttons
	local MemberMax = 0;
	local MemberMin = 5;
	if ( BeneCastState[UnitName('player')].playerasdefault ) then
		value = BeneCastState[UnitName('player')][1].buttonsize;
	end
	for i = MemberMin, MemberMax do
		for j = 1,MAX_NUMBER_OF_BUTTONS do
			-- Break out of the j loop if the member at i is not a of the same class
			if ( UnitExists(_targets[i]) and _targets[i] ~= 'player' ) then
				if ( parentID == _classes[UnitClass(_targets[i])] and not BeneCastState[UnitName('player')].playerasdefault and _targets[i] ~= 'player' ) then
					-- If BeneCastMemberiButtonj is exists then
					if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
						local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
						-- Set Button height and width to value
						button:SetHeight(value);
						button:SetWidth(value);
						local buttonborder = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'NormalTexture');
						-- Set the Button border to fit around the button
						buttonborder:SetHeight(value * 2 - (value/36) * 10);
						buttonborder:SetWidth(value * 2 - (value/36) * 10);
						local buttoncooldown = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Cooldown');
						-- Set Cooldown size
						buttoncooldown:SetScale((value / 36) * .75);
						local buttonflash = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Flash');
						-- Set Flash size
						buttonflash:SetHeight(value * 2 - (value/36) * 10);
						buttonflash:SetWidth(value * 2 - (value/36) * 10);
						-- Resize the BeneCastMemberFrame based on the number of buttons shown
						if ( button:IsVisible() ) then
							local frame = getglobal('BeneCastMember' .. i);
							frame:SetWidth(value * j + 11 + (3 * (j - 1)));
							frame:SetHeight(value + 11);
						end
					end
				end
			elseif ( parentID == 1 and _targets[i] == 'player' ) then
				if ( getglobal('BeneCastMember' .. i .. 'Button' .. j) ) then
					local button = getglobal('BeneCastMember' .. i .. 'Button' .. j);
					-- Set Button height and width to value
					button:SetHeight(value);
					button:SetWidth(value);
					local buttonborder = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'NormalTexture');
					-- Set the Button border to fit around the button
					buttonborder:SetHeight(value * 2 - (value/36) * 10);
					buttonborder:SetWidth(value * 2 - (value/36) * 10);
					local buttoncooldown = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Cooldown');
					-- Set Cooldown size
					buttoncooldown:SetScale((value / 36) * .75);
					local buttonflash = getglobal('BeneCastMember' .. i .. 'Button' .. j .. 'Flash');
					-- Set Flash size
					buttonflash:SetHeight(value * 2 - (value/36) * 10);
					buttonflash:SetWidth(value * 2 - (value/36) * 10);
					-- Resize the BeneCastMemberFrame based on the number of buttons shown
					if ( button:IsVisible() ) then
						local frame = getglobal('BeneCastMember' .. i);
						frame:SetWidth(value * j + 11 + (3 * (j - 1)));
						frame:SetHeight(value + 11);
					end
				end
			end
		end
	end
	
	-- Set the slider value to that of value
	this:SetValue(value);
	
end

--
-- BeneCastButtonSizeSlider Game Event Handlers
--

-- Sets up the text for a the button number slider
function BeneCast_ButtonNumberSliderLoad()

	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this.tooltipText = STRING_CONSTANTS.TEXT_BUTTONNUMBERSLIDER_TOOLTIP;
	getglobal(this:GetName() .. 'ButtonSizeSliderText'):SetText(STRING_CONSTANTS.TEXT_BUTTON_NUMBER);
	
end

-- Resizes BeneCast button number
function BeneCast_ButtonNumberResize()

	-- Return if BeneCastState or the player name has not been loaded
	if ( not UnitName('player') or not BeneCastState ) then
		return;
	end
	
	-- Set value to the slider value
	local value = this:GetValue();
	
	-- Save the new value into BeneCastState
	BeneCastState[UnitName('player')].numberofbuttons = value;
	
	getglobal(this:GetName() .. 'ButtonSizeSliderValue'):SetText(value);
	
end

-- Loads BeneCast button number
function BeneCast_ButtonNumberLoad()

	-- Return if BeneCastState or the player name has not been loaded
	if ( not UnitName('player') or not BeneCastState ) then
		return;
	end
	
	-- Return if BeneCastState['player'] has not yet been created
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	
	-- Set value to the saved size in BeneCastState
	local value = BeneCastState[UnitName('player')].numberofbuttons;
	
	-- Set the slider value to that of value
	this:SetValue(value);
	
end

--
-- BeneCastOverHealSlider Game Event Handlers
--

-- Sets up the text for a the button size sliders
function BeneCast_OverhealSliderLoad()

	this:RegisterEvent("UNIT_MODEL_CHANGED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this.tooltipText = STRING_CONSTANTS.TEXT_OVERHEALSLIDER_TOOLTIP;
	getglobal(this:GetName() .. 'OverhealSliderText'):SetText(STRING_CONSTANTS.TEXT_RANKS_TO_OVERHEAL);
	
end

-- Resizes BeneCast overheal amount
function BeneCast_OverhealResize()

	-- Return if BeneCastState or the player name has not been loaded
	if ( not UnitName('player') or not BeneCastState ) then
		return;
	end
	
	-- Set value to the slider value
	local value = this:GetValue();
	
	-- Save the new value into BeneCastState
	BeneCastState[UnitName('player')].overhealamount = value;
	
	getglobal(this:GetName() .. 'OverhealSliderValue'):SetText(value);
	
end

-- Loads BeneCast overheal amount
function BeneCast_OverhealLoad()

	-- Return if BeneCastState or the player name has not been loaded
	if ( not UnitName('player') or not BeneCastState ) then
		return;
	end
	
	-- Return if BeneCastState['player'] has not yet been created
	if ( not BeneCastState[UnitName('player')] ) then
		return;
	end
	
	-- Set value to the saved size in BeneCastState
	local value = BeneCastState[UnitName('player')].overhealamount;
	
	-- Set the slider value to that of value
	this:SetValue(value);
	
end

--
-- BeneCastFrame Game Event Handlers
--

-- BeneCastFrame OnLoad function
function BeneCastFrame_OnLoad()

	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("SPELLS_CHANGED");
	this:RegisterEvent("SPELLCAST_FAILED");
	this:RegisterEvent("SPELLCAST_INTERRUPTED");
	this:RegisterEvent("SPELLCAST_STOP");
	this:RegisterEvent("SPELLCAST_START");
	this:RegisterEvent("VARIABLES_LOADED");
	-- Frame Level cleanup to maintain good performance
	this:SetFrameLevel(1);
	BeneCastFrame_ShowSubFrame('BeneCastPlayerFrame');
	getglobal('BeneCastFrameTab1'):SetChecked(1);
	getglobal('BeneCastFrameTab1'):Disable();
	BINDING_HEADER_BENECAST = 'BeneCast';
	BINDING_NAME_BENECAST_OPTIONS = BeneCast_Constants.TEXT_BENECAST_OPTIONS;
	BINDING_NAME_BENECAST_TOGGLE_BUFFS = BeneCast_Constants.TEXT_BENECAST_TOGGLE_BUFFS;
	BINDING_NAME_BENECAST_BUTTON1 = BeneCast_Constants.TEXT_BENECAST_BUTTON1;
	BINDING_NAME_BENECAST_BUTTON2 = BeneCast_Constants.TEXT_BENECAST_BUTTON2;
	BINDING_NAME_BENECAST_BUTTON3 = BeneCast_Constants.TEXT_BENECAST_BUTTON3;
	BINDING_NAME_BENECAST_BUTTON4 = BeneCast_Constants.TEXT_BENECAST_BUTTON4;
	BINDING_NAME_BENECAST_BUTTON5 = BeneCast_Constants.TEXT_BENECAST_BUTTON5;
	BINDING_NAME_BENECAST_BUTTON6 = BeneCast_Constants.TEXT_BENECAST_BUTTON6;
	BINDING_NAME_BENECAST_BUTTON7 = BeneCast_Constants.TEXT_BENECAST_BUTTON7;
	BINDING_NAME_BENECAST_BUTTON8 = BeneCast_Constants.TEXT_BENECAST_BUTTON8;
	BINDING_NAME_BENECAST_BUTTON9 = BeneCast_Constants.TEXT_BENECAST_BUTTON9;
	BINDING_NAME_BENECAST_BUTTON10 = BeneCast_Constants.TEXT_BENECAST_BUTTON10;
	SLASH_BENECAST1 = "/benecast";
	SLASH_BENECAST2 = "/bc";
	SlashCmdList["BENECAST"] = function(msg)
		BeneCast_SlashCommandHandler(msg);
	end

end

-- BeneCastFrame OnShow function
function BeneCastFrame_OnShow()

	getglobal("BeneCastFrameTab1NormalTexture"):SetTexture("Interface\\Icons\\Spell_Nature_WispSplode");
	getglobal("BeneCastFrameTab1").tooltip = UnitName('player');
	
	getglobal("BeneCastFrameTab2NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab2NormalTexture"):SetTexCoord(0.7421875, 0.98828125, 0, 0.25);
	getglobal("BeneCastFrameTab2").tooltip = STRING_CONSTANTS.CLASS_DRUID;
	
	getglobal("BeneCastFrameTab3NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab3NormalTexture"):SetTexCoord(0, 0.25, 0.25, 0.5);
	getglobal("BeneCastFrameTab3").tooltip = STRING_CONSTANTS.CLASS_HUNTER;
	
	getglobal("BeneCastFrameTab4NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab4NormalTexture"):SetTexCoord(0.25, 0.49609375, 0, 0.25);
	getglobal("BeneCastFrameTab4").tooltip = STRING_CONSTANTS.CLASS_MAGE;
	
	getglobal("BeneCastFrameTab5NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab5NormalTexture"):SetTexCoord(0.49609375, 0.7421875, 0.25, 0.5);
	getglobal("BeneCastFrameTab5").tooltip = STRING_CONSTANTS.CLASS_PRIEST;
	
	getglobal("BeneCastFrameTab6NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab6NormalTexture"):SetTexCoord(0.49609375, 0.7421875, 0, 0.25);
	getglobal("BeneCastFrameTab6").tooltip = STRING_CONSTANTS.CLASS_ROGUE;
	
	getglobal("BeneCastFrameTab7NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab7NormalTexture"):SetTexCoord(0.7421875, 0.98828125, 0.25, 0.5);
	getglobal("BeneCastFrameTab7").tooltip = STRING_CONSTANTS.CLASS_WARLOCK;
	
	getglobal("BeneCastFrameTab8NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
	getglobal("BeneCastFrameTab8NormalTexture"):SetTexCoord(0, 0.25, 0, 0.25);
	getglobal("BeneCastFrameTab8").tooltip = STRING_CONSTANTS.CLASS_WARRIOR;
	
	if ( UnitFactionGroup('player') == STRING_CONSTANTS.ALLIANCE ) then
		getglobal("BeneCastFrameTab9NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
		getglobal("BeneCastFrameTab9NormalTexture"):SetTexCoord(0.0, 0.25, 0.5, 0.75);
		getglobal("BeneCastFrameTab9").tooltip = STRING_CONSTANTS.CLASS_PALADIN;
	else
		getglobal("BeneCastFrameTab9NormalTexture"):SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
		getglobal("BeneCastFrameTab9NormalTexture"):SetTexCoord(0.25, 0.49609375, 0.25, 0.5);
		getglobal("BeneCastFrameTab9").tooltip = STRING_CONSTANTS.CLASS_SHAMAN;
	end
	
	getglobal("BeneCastFrameTab10NormalTexture"):SetTexture("Interface\\Icons\\INV_Misc_Wrench_01");
	getglobal("BeneCastFrameTab10").tooltip = STRING_CONSTANTS.TEXT_SETUP;

end

-- OnEvent for BeneCastFrame, only called once each time
function BeneCastFrame_OnEvent(event, arg1)

	if ( event == 'PLAYER_ENTERING_WORLD' ) then
		-- Set _class to the player's class
		_class = UnitClass('player');
		
		-- If the player's class is not in _spell_types then unregister the events and return
		if ( _class and not _spell_types[_class] ) then
			BeneCastMinimapButton:Hide();
			return;
		end
		
		-- Load up the Spell data
		BeneCast_ResetData();
	-- The following is really, really confusing
	-- If SPELLCAST_FAILED then a BeneCast spell may have failed
	elseif ( event == 'SPELLCAST_FAILED' ) then
		-- Return if BeneCastState has not been set up for this player
		if ( not BeneCastState[UnitName('player')] ) then
			return;
		end
		-- If the player can't use BeneCast then return
		if ( not _spell_types[_class] ) then
			return;
		end
		-- A BeneCast spell failed if a _spellstarted and _sentmessage
		if ( _spellstarted and _sentmessage ) then
			BeneCast_PartyNotifyFail('SPELLCAST_FAILED');
		end
		-- Clear out everything used for party notification just in case
		_spellstarted = nil;
		_spellstopped = nil;
		_spelltarget = nil;
		_spellbeingcast = nil;
		_spellismax = nil;
		_sentmessage = nil;
	-- If SPELLCAST_INTERRUPTED then a BeneCast spell may have been interrupted
	elseif ( event == 'SPELLCAST_INTERRUPTED' ) then
		-- Return if BeneCastState has not been set up for this player
		if ( not BeneCastState[UnitName('player')] ) then
			return;
		end
		-- If the player can't use BeneCast then return
		if ( not _spell_types[_class] ) then
			return;
		end
		-- A BeneCast spell was interrupted by the player if _spellstopped and _sentmessage
		if ( _spellstopped and _sentmessage ) then
			BeneCast_PartyNotifyFail('SPELLCAST_CANCELED');
		end
		-- A BeneCast spell was interrupted by something else if _spellstarted and _sentmessage
		if ( _spellstarted and _sentmessage ) then
			BeneCast_PartyNotifyFail('SPELLCAST_INTERRUPTED');
		end
		-- Clear out everything just in case
		_spellstarted = nil;
		_spellstopped = nil;
		_spelltarget = nil;
		_spellbeingcast = nil;
		_spellismax = nil;
		_sentmessage = nil;
	elseif ( event == 'SPELLCAST_STOP' ) then
		-- Return if BeneCastState has not been set up for this player
		if ( not BeneCastState[UnitName('player')] ) then
			return;
		end
		-- If the player can't use BeneCast then return
		if ( not _spell_types[_class] ) then
			return;
		end
		-- If _spellbeingcast and notification is on then this is an instant spell and you should do notification
		if ( _spellbeingcast and BeneCastState[UnitName('player')].notify ) then
			-- _sentmessage will be set to true of BeneCast_PartyNotify sends a message
			_sentmessage = BeneCast_PartyNotify(_spelltarget, _spellbeingcast, _spellismax, 0);
		-- Otherwise set _setmessage to false
		else
			_sentmessage = nil
		end
		-- If _spellstarted, then a BeneCast spell was started and you should set _sentmessage and _spellstopped to true to indicate that the spell was interrupted by the user
		if ( _spellstarted ) then
			_sentmessage = true;
			_spellstopped = true;
			_spellstarted = nil;
		else
			-- Clear this, just in case
			_spellstopped = nil;
		end
		-- Clear these out just in case
		_spelltarget = nil;
		_spellbeingcast = nil;
		_spellismax = nil;
	elseif ( event == 'SPELLCAST_START' ) then
		-- Return if BeneCastState has not been set up for this player
		if ( not BeneCastState[UnitName('player')] ) then
			return;
		end
		-- If the player can't use BeneCast then return
		if ( not _spell_types[_class] ) then
			return;
		end
		-- If the spell being cast exists in _spell_types, notification is on, and there is a _spellbeingcast then do notification
		if ( _spell_types[_class][arg1] and BeneCastState[UnitName('player')].notify and _spellbeingcast ) then
			-- Set _sentmessage to true of BeneCast_PartyNotify sent a message
			_sentmessage = BeneCast_PartyNotify(_spelltarget, _spellbeingcast, _spellismax, arg2);
			-- Clear out _spelltarget, _spellbeingcast, and _spellismax so it isn't used by SPELLCAST_STOP
			_spelltarget = nil;
			_spellbeingcast = nil;
			_spellismax = nil;
			-- If you sent a message then _spellstarted to indicate failures/interruptions should send notifications
			if ( _sentmessage ) then
				_spellstarted = true;
			end
		else
			-- Clear out everything just in case
			_spellstarted = nil;
			_spellstopped = nil;
			_spelltarget = nil;
			_spellbeingcast = nil;
			_spellismax = nil;
			_sentmessage = nil;
		end
	-- We only want spell changes after entering the world
	elseif ( event == 'SPELLS_CHANGED' ) then
		-- Return if you can't use BeneCast
		if ( not _spell_types[_class] ) then
			return;
		end
		
		-- Reset most of the tables for reloading of spells and reload the spell data
		BeneCast_ResetData();
	elseif ( event == 'VARIABLES_LOADED' ) then
		-- If there is no BeneCastState or it is outdated crate a new one
		if ( not BeneCastState or not BeneCastState.version or BeneCastState.version < 1.15 ) then
			BeneCastState = {};
			BeneCastState.version = 1.15;
			BeneCastState.hideminimapbutton = nil;
		end
		-- If BeneCastState does exist then make sure the Minimap button is hidden if it's set to be hidden
		if ( BeneCastState ) then
			if ( BeneCastState.hideminimapbutton ) then
				getglobal('BeneCastMinimapButton'):Hide();
			else
				getglobal('BeneCastMinimapButton'):Show();
			end
		end
	end
	
end

--
-- BeneCastMiniMapButton Game Event Handlers
--

-- Clicks on the BeneCast button do different things
function BeneCast_FrameToggle(mouseButton)

	local playername = UnitName('player');

	-- If this player does not have saved settings make them
	if ( not BeneCastState[playername] ) then
		BeneCastState[playername] = {};
		BeneCastState[playername].notify = nil;
		BeneCastState[playername].notifyself = nil;
		BeneCastState[playername].notifyparty = nil;
		BeneCastState[playername].notifyraid = nil;
		BeneCastState[playername].notifytarget = nil;
		BeneCastState[playername].notifyrank = true;
		BeneCastState[playername].notifytime = true;
		BeneCastState[playername].notifyshowself = true;
		BeneCastState[playername].notifyheal = true;
		BeneCastState[playername].notifymaxheal = nil;
		BeneCastState[playername].notifycure = true;
		BeneCastState[playername].notifybuff = true;
		BeneCastState[playername].maxheal = nil;
		BeneCastState[playername].playerasdefault = true;
		BeneCastState[playername].numberofbuttons = 6;
		BeneCastState[playername].tooltips = true;
		BeneCastState[playername].tooltipnameonly = nil;
		BeneCastState[playername].overheal = nil;
		BeneCastState[playername].overhealamount = 3;
		BeneCastState[playername].showallbuffs = nil;
		BeneCastState[playername].unlock = nil;
		for i = 1, 9 do
			BeneCastState[playername][i] = {};
			BeneCastState[playername][i].buttonsize = 36;
		end
		for i in _spell_buttons do
			BeneCastState[playername][i] = {};
				BeneCastState[playername][i][1] = nil;
				BeneCastState[playername][i][2] = nil;
				BeneCastState[playername][i][3] = nil;
				BeneCastState[playername][i][4] = nil;
				BeneCastState[playername][i][5] = nil;
				BeneCastState[playername][i][6] = nil;
				BeneCastState[playername][i][7] = nil;
				BeneCastState[playername][i][8] = nil;
				BeneCastState[playername][i][9] = nil;
		end
	end
	
	-- If the mouseButton is the left button toggle BeneCastFrame's visibility
	if ( mouseButton == 'LeftButton' ) then
		local frame = getglobal('BeneCastFrame');
		if ( frame:IsVisible() ) then
			frame:Hide();
		else
			frame:Show();
		end
	end
	
end